<?php



namespace Orbipay\PaymentsApi\Client\Api;

use GuzzleHttp\Client;
use GuzzleHttp\ClientInterface;
use GuzzleHttp\Exception\RequestException;
use GuzzleHttp\Psr7\MultipartStream;
use GuzzleHttp\Psr7\Request;
use GuzzleHttp\RequestOptions;
use Orbipay\PaymentsApi\Client\ApiException;
use Orbipay\PaymentsApi\Client\Configuration;
use Orbipay\PaymentsApi\Client\HeaderSelector;
use Orbipay\PaymentsApi\Client\ObjectSerializer;
use Psr\Http\Message\ResponseInterface;
use Orbipay\PaymentsApi\Client\CommonUtil;
use Orbipay\PaymentsApi\Client\AuthenticationUtil;
use GuzzleHttp\Exception\ConnectException;
use lf4php\LoggerFactory;

/**
 * Warning:
 *
 * <p><font color="red"> This class subject to change without prior notice, Please dont use this class directly. </font></p>


 *
 * @category Class
 * @package  Orbipay\PaymentsApi\Client\Api
 */
class PaymentApi
{
    /**
     * @var ClientInterface
     */
    protected $client;

    /**
     * @var Configuration
     */
    protected $config;

    /**
     * @var HeaderSelector
     */
    protected $headerSelector;

    /**
     * @param ClientInterface $client
     * @param Configuration   $config
     * @param HeaderSelector  $selector
     */
    public function __construct(
        ClientInterface $client = null,
        Configuration $config = null,
        HeaderSelector $selector = null
    ) {
        $this->client = $client ?: new Client();
        $this->config = $config ?: new Configuration();
        $this->headerSelector = $selector ?: new HeaderSelector();
    }

    /**
     * @return Configuration
     */
    public function getConfig()
    {
        return $this->config;
    }

    /**
     * Operation createPayment
     *
     * Create Payment
     *
     * @param  string $channel The channel through which the API is invoked. (required)
     * @param  string $client_key The unique identifier assigned by EBPP to the client. (required)
     * @param  string $product The product identifier corresponding to the API. (required)
     * @param  string $timestamp The timestamp for the moment when the API request is created. (required)
     * @param  string $idempotent_request_key The unique token that clients can generate and maintain in order to identify an API request. (required)
     * @param  string $requestor_type Type of the requestor of the API. (required)
     * @param  string $id_customer The unique identifier assigned by EBPP to the customer. (required)
     * @param  CreatePaymentRequest $create_payment_request JSON containing all the fields required to make a payment. (required)
     * @param  string $requestor The identifier for the requestor of the API. (optional)
     * @param  string $x_opay_headers Intended for the future use. (optional)
     * @param  string $trace_id The unique reference that can be used for tracing and debugging an API call. (optional)
     *
     * @throws \Orbipay\PaymentsApi\Client\ApiException on non-2xx response
     * @throws \InvalidArgumentException
     * @return array of \Orbipay\PaymentsApi\Client\Model\PaymentResponse, HTTP status code, HTTP response headers (array of strings)
     */
    public function createPayment($channel, $client_key, $product, $timestamp, $idempotent_request_key, $requestor_type, $id_customer, $create_payment_request, $requestor = null, $x_opay_headers = null, $trace_id = null)
    {
        $response = $this->createPaymentWithHttpInfo($channel, $client_key, $product, $timestamp, $idempotent_request_key, $requestor_type, $id_customer, $create_payment_request, $requestor, $x_opay_headers, $trace_id);
        return $response;
    }

    /**
     * Operation createPaymentWithHttpInfo
     *
     * Create Payment
     *
     * @param  string $channel The channel through which the API is invoked. (required)
     * @param  string $client_key The unique identifier assigned by EBPP to the client. (required)
     * @param  string $product The product identifier corresponding to the API. (required)
     * @param  string $timestamp The timestamp for the moment when the API request is created. (required)
     * @param  string $idempotent_request_key The unique token that clients can generate and maintain in order to identify an API request. (required)
     * @param  string $requestor_type Type of the requestor of the API. (required)
     * @param  string $id_customer The unique identifier assigned by EBPP to the customer. (required)
     * @param  CreatePaymentRequest $create_payment_request JSON containing all the fields required to make a payment. (required)
     * @param  string $requestor The identifier for the requestor of the API. (optional)
     * @param  string $x_opay_headers Intended for the future use. (optional)
     * @param  string $trace_id The unique reference that can be used for tracing and debugging an API call. (optional)
     *
     * @throws \Orbipay\PaymentsApi\Client\ApiException on non-2xx response
     * @throws \InvalidArgumentException
     * @return array of \Orbipay\PaymentsApi\Client\Model\PaymentResponse, HTTP status code, HTTP response headers (array of strings)
     */
    public function createPaymentWithHttpInfo($channel, $client_key, $product, $timestamp, $idempotent_request_key, $requestor_type, $id_customer, $create_payment_request, $requestor = null, $x_opay_headers = null, $trace_id = null)
    {
        $returnType = '\Orbipay\PaymentsApi\Client\Model\PaymentResponse';
        list($request, $_tempBody, $formParams, $masked_authentication_data) = $this->createPaymentRequest($channel, $client_key, $product, $timestamp, $idempotent_request_key, $requestor_type, $id_customer, $create_payment_request, $requestor, $x_opay_headers, $trace_id);
		$logger = LoggerFactory::getLogger(__CLASS__);
        try {
            $options = $this->createHttpClientOption();
            try {
             	$isTimedOut = false;
             	if($logger->isDebugEnabled()){
             		$logger->debug('Connect TimeOut :'.$options[RequestOptions::TIMEOUT]);
             		$logger->debug('Read TimeOut :'.$options[RequestOptions::TIMEOUT]);
             	}
             	$logger->info('Sending Request to Service');
                $response = $this->client->send($request, $options);
            } catch (RequestException $e) {
            	$logger->info('Receive Response from Service : HTTP Status Code :'.$e->getCode());
            	if($e instanceof ConnectException){
					$logger->warn("[{$e->getCode()}] {$e->getMessage()}");
                    $isTimedOut = true;
                }
                throw new ApiException(
                    "[{$e->getCode()}] {$e->getMessage()}",
                    $e->getCode(),
                    $e->getResponse() ? $e->getResponse()->getHeaders() : null,
                    $e->getResponse() ? $e->getResponse()->getBody()->getContents() : null, CommonUtil::get_masked_string(CommonUtil::wrapHttpRequest($request, $_tempBody, $formParams)), $masked_authentication_data
                ,$isTimedOut);
            }

            $statusCode = $response->getStatusCode();
			$logger->info('Receive Response from Service : HTTP Status Code :'.$statusCode);
            if ($statusCode < 200 || $statusCode > 299) {
                throw new ApiException(
                    sprintf(
                        '[%d] Error connecting to the API (%s)',
                        $statusCode,
                        $request->getUri()
                    ),
                    $statusCode,
                    $response->getHeaders(),
                    $response->getBody(), CommonUtil::get_masked_string(CommonUtil::wrapHttpRequest($request, $_tempBody, $formParams)), $masked_authentication_data
                );
            }

            $responseBody = $response->getBody();
            if ($returnType === '\SplFileObject') {
                $content = $responseBody; //stream goes to serializer
            } else {
                $content = $responseBody->getContents();
                if ($returnType !== 'string') {
                    $content = json_decode($content);
                }
            }

            return [
                ObjectSerializer::deserialize($content, $returnType, []),
                $response->getStatusCode(),
                $response->getHeaders()
            ];

        } catch (ApiException $e) {
            switch ($e->getCode()) {
                case 201:
                    $data = ObjectSerializer::deserialize(
                        json_decode($e->getResponseBody()),
                        '\Orbipay\PaymentsApi\Client\Model\PaymentResponse',
                        $e->getResponseHeaders()
                    );
                    $e->setResponseObject($data);
                    break;
                case 400:
                    $data = ObjectSerializer::deserialize(
                        json_decode($e->getResponseBody()),
                        '\Orbipay\PaymentsApi\Client\Model\ErrorResponseVo',
                        $e->getResponseHeaders()
                    );
                    $e->setResponseObject($data);
                    break;
                case 409:
                    $data = ObjectSerializer::deserialize(
                        json_decode($e->getResponseBody()),
                        '\Orbipay\PaymentsApi\Client\Model\ErrorResponseVo',
                        $e->getResponseHeaders()
                    );
                    $e->setResponseObject($data);
                    break;
                case 422:
                    $data = ObjectSerializer::deserialize(
                        json_decode($e->getResponseBody()),
                        '\Orbipay\PaymentsApi\Client\Model\ErrorResponseVo',
                        $e->getResponseHeaders()
                    );
                    $e->setResponseObject($data);
                    break;
            }
            throw $e;
        }
    }


    /**
     * Create request for operation 'createPayment'
     *
     * @param  string $channel The channel through which the API is invoked. (required)
     * @param  string $client_key The unique identifier assigned by EBPP to the client. (required)
     * @param  string $product The product identifier corresponding to the API. (required)
     * @param  string $timestamp The timestamp for the moment when the API request is created. (required)
     * @param  string $idempotent_request_key The unique token that clients can generate and maintain in order to identify an API request. (required)
     * @param  string $requestor_type Type of the requestor of the API. (required)
     * @param  string $id_customer The unique identifier assigned by EBPP to the customer. (required)
     * @param  CreatePaymentRequest $create_payment_request JSON containing all the fields required to make a payment. (required)
     * @param  string $requestor The identifier for the requestor of the API. (optional)
     * @param  string $x_opay_headers Intended for the future use. (optional)
     * @param  string $trace_id The unique reference that can be used for tracing and debugging an API call. (optional)
     *
     * @throws \InvalidArgumentException
     * @return array
     */
    protected function createPaymentRequest($channel, $client_key, $product, $timestamp, $idempotent_request_key, $requestor_type, $id_customer, $create_payment_request, $requestor = null, $x_opay_headers = null, $trace_id = null)
    {
    
     $logger = LoggerFactory::getLogger(__CLASS__);
    
        // verify the required parameter 'id_customer' is set
        if ($id_customer === null || (is_array($id_customer) && count($id_customer) === 0) || strlen($id_customer) === 0) {
     		$logger->error('Missing the required parameter $id_customer');
           throw new \InvalidArgumentException(
                 'Missing the required parameter $id_customer'
             );
        }
       
		$method = 'POST';
        $resourcePath = '/customers/{ID_CUSTOMER}/payments';
        $formParams = [];
        $queryParams = [];
        $headerParams = [];
        $httpBody = '';
        $multipart = false;
        $masked_authentication_data = '';

        // header params
        if ($channel !== null) {
            $headerParams['channel'] = ObjectSerializer::toHeaderValue($channel);
        }
        // header params
        if ($client_key !== null) {
            $headerParams['client_key'] = ObjectSerializer::toHeaderValue($client_key);
        }
        // header params
        if ($product !== null) {
            $headerParams['product'] = ObjectSerializer::toHeaderValue($product);
        }
        // header params
        if ($timestamp !== null) {
            $headerParams['timestamp'] = ObjectSerializer::toHeaderValue($timestamp);
        }
        // header params
        if ($idempotent_request_key !== null) {
            $headerParams['idempotent_request_key'] = ObjectSerializer::toHeaderValue($idempotent_request_key);
        }
        // header params
        if ($requestor_type !== null) {
            $headerParams['requestor_type'] = ObjectSerializer::toHeaderValue($requestor_type);
        }
        // header params
        if ($requestor !== null) {
            $headerParams['requestor'] = ObjectSerializer::toHeaderValue($requestor);
        }
        // header params
        if ($x_opay_headers !== null) {
            $headerParams['X-OPAY-Headers'] = ObjectSerializer::toHeaderValue($x_opay_headers);
        }
        // header params
        if ($trace_id !== null) {
            $headerParams['trace_id'] = ObjectSerializer::toHeaderValue($trace_id);
        }

        // path params
        if ($id_customer !== null) {
            $resourcePath = str_replace(
                '{' . 'ID_CUSTOMER' . '}',
                ObjectSerializer::toPathValue($id_customer),
                $resourcePath
            );
        }

        // body params
        $_tempBody = null;
        if (isset($create_payment_request)) {
            $_tempBody = $create_payment_request;
        }

        if ($multipart) {
            $headers = $this->headerSelector->selectHeadersForMultipart(
                ['application/json']
            );
        } else {
            $headers = $this->headerSelector->selectHeaders(
                ['application/json'],
                ['application/json']
            );
        }

        // for model (json/xml)
        if (isset($_tempBody)) {
            // $_tempBody is the method argument, if present
            $httpBody = $_tempBody;
            // \stdClass has no __toString(), so we should encode it manually
            if ($httpBody instanceof \stdClass && $headers['Content-Type'] === 'application/json') {
                $httpBody = \GuzzleHttp\json_encode($httpBody);
            }
        } elseif (count($formParams) > 0) {
            if ($multipart) {
                $multipartContents = [];
                foreach ($formParams as $formParamName => $formParamValue) {
                    $multipartContents[] = [
                        'name' => $formParamName,
                        'contents' => $formParamValue
                    ];
                }
                // for HTTP post (form)
                $httpBody = new MultipartStream($multipartContents);

            } elseif ($headers['Content-Type'] === 'application/json') {
                $httpBody = \GuzzleHttp\json_encode($formParams);

            } else {
                // for HTTP post (form)
                $httpBody = \GuzzleHttp\Psr7\build_query($formParams);
            }
        }
        
        if($logger->isDebugEnabled()){
            $logger->debug('HTTP Method     :'.$method);
            $logger->debug('HTTP URL        :'.$this->config->getHost() . $resourcePath);
        }
        
         if(empty($this->getConfig()->getClientApiKey())){
        	$this->getConfig()->setClientApiKey($this->getConfig()->getClientKey());
        }
        
        if(empty($this->getConfig()->getAuthScheme()) || $this->getConfig()->authScheme1 == $this->getConfig()->getAuthScheme()){
	         
	        list($hash, $masked_authentication_data) = AuthenticationUtil::computeOPAY1HMACSHA256Hash($method, $resourcePath, $headerParams, $queryParams, $formParams, $_tempBody, $this->getConfig()->getClientSignatureKey());
	        $headerParams['Authorization'] = $this->getConfig()->authScheme1 . " Credential=" . $this->getConfig()->getClientApiKey() . ",Signature=" . $hash;
        
        }elseif($this->getConfig()->authScheme2 == $this->getConfig()->getAuthScheme()){
				
	        $hash = AuthenticationUtil::computeOPAY2HMACSHA256Hash($this->getConfig()->getClientKey(), $this->getConfig()->getClientApiKey() ,$this->getConfig()->getClientSignatureKey(), $headerParams['timestamp']);
	        $headerParams['Authorization'] = $this->getConfig()->authScheme2 . " Credential=" . $this->getConfig()->getClientApiKey() . ",Signature=" . $hash;
	                
        }elseif($this->getConfig()->authScheme3 == $this->getConfig()->getAuthScheme()){
        	$headerParams['Authorization'] = $this->getConfig()->authScheme3 . " Credential=" . $this->getConfig()->getClientApiKey();
        }else{
        	$logger->error('Invalid auth_scheme received.');
           throw new \InvalidArgumentException('Invalid auth_scheme received.');
        }
        


        $defaultHeaders = [];
        if ($this->config->getUserAgent()) {
            $defaultHeaders['User-Agent'] = $this->config->getUserAgent();
        }

        $headers = array_merge(
            $defaultHeaders,
            $headerParams,
            $headers
        );

		if($logger->isDebugEnabled()){
			$logger->debug('Request Headers  :'.json_encode($headers));
		}
		
        $query = \GuzzleHttp\Psr7\build_query($queryParams);
        return [ new Request(
            $method,
            $this->config->getHost() . $resourcePath . ($query ? "?{$query}" : ''),
            $headers,
            $httpBody
        ), $_tempBody, $formParams, $masked_authentication_data];
    }

    /**
     * Operation deletePayment
     *
     * Delete Payment
     *
     * @param  string $channel The channel through which the API is invoked. (required)
     * @param  string $client_key The unique identifier assigned by EBPP to the client. (required)
     * @param  string $product The product identifier corresponding to the API. (required)
     * @param  string $timestamp The timestamp for the moment when the API request is created. (required)
     * @param  string $idempotent_request_key The unique token that clients can generate and maintain in order to identify an API request. (required)
     * @param  string $requestor_type Type of the requestor of the API. (required)
     * @param  string $id_payment The unique identifier assigned by EBPP to the payment. (required)
     * @param  string $id_customer The unique identifier assigned by EBPP to the customer. (required)
     * @param  DeletePaymentRequest $delete_payment_request Details of payment to be deleted/cancelled (required)
     * @param  string $requestor The identifier for the requestor of the API. (optional)
     * @param  string $x_opay_headers Intended for the future use. (optional)
     * @param  string $trace_id The unique reference that can be used for tracing and debugging an API call. (optional)
     *
     * @throws \Orbipay\PaymentsApi\Client\ApiException on non-2xx response
     * @throws \InvalidArgumentException
     * @return array of \Orbipay\PaymentsApi\Client\Model\DeleteResponse, HTTP status code, HTTP response headers (array of strings)
     */
    public function deletePayment($channel, $client_key, $product, $timestamp, $idempotent_request_key, $requestor_type, $id_payment, $id_customer, $delete_payment_request, $requestor = null, $x_opay_headers = null, $trace_id = null)
    {
        $response = $this->deletePaymentWithHttpInfo($channel, $client_key, $product, $timestamp, $idempotent_request_key, $requestor_type, $id_payment, $id_customer, $delete_payment_request, $requestor, $x_opay_headers, $trace_id);
        return $response;
    }

    /**
     * Operation deletePaymentWithHttpInfo
     *
     * Delete Payment
     *
     * @param  string $channel The channel through which the API is invoked. (required)
     * @param  string $client_key The unique identifier assigned by EBPP to the client. (required)
     * @param  string $product The product identifier corresponding to the API. (required)
     * @param  string $timestamp The timestamp for the moment when the API request is created. (required)
     * @param  string $idempotent_request_key The unique token that clients can generate and maintain in order to identify an API request. (required)
     * @param  string $requestor_type Type of the requestor of the API. (required)
     * @param  string $id_payment The unique identifier assigned by EBPP to the payment. (required)
     * @param  string $id_customer The unique identifier assigned by EBPP to the customer. (required)
     * @param  DeletePaymentRequest $delete_payment_request Details of payment to be deleted/cancelled (required)
     * @param  string $requestor The identifier for the requestor of the API. (optional)
     * @param  string $x_opay_headers Intended for the future use. (optional)
     * @param  string $trace_id The unique reference that can be used for tracing and debugging an API call. (optional)
     *
     * @throws \Orbipay\PaymentsApi\Client\ApiException on non-2xx response
     * @throws \InvalidArgumentException
     * @return array of \Orbipay\PaymentsApi\Client\Model\DeleteResponse, HTTP status code, HTTP response headers (array of strings)
     */
    public function deletePaymentWithHttpInfo($channel, $client_key, $product, $timestamp, $idempotent_request_key, $requestor_type, $id_payment, $id_customer, $delete_payment_request, $requestor = null, $x_opay_headers = null, $trace_id = null)
    {
        $returnType = '\Orbipay\PaymentsApi\Client\Model\DeleteResponse';
        list($request, $_tempBody, $formParams, $masked_authentication_data) = $this->deletePaymentRequest($channel, $client_key, $product, $timestamp, $idempotent_request_key, $requestor_type, $id_payment, $id_customer, $delete_payment_request, $requestor, $x_opay_headers, $trace_id);
		$logger = LoggerFactory::getLogger(__CLASS__);
        try {
            $options = $this->createHttpClientOption();
            try {
             	$isTimedOut = false;
             	if($logger->isDebugEnabled()){
             		$logger->debug('Connect TimeOut :'.$options[RequestOptions::TIMEOUT]);
             		$logger->debug('Read TimeOut :'.$options[RequestOptions::TIMEOUT]);
             	}
             	$logger->info('Sending Request to Service');
                $response = $this->client->send($request, $options);
            } catch (RequestException $e) {
            	$logger->info('Receive Response from Service : HTTP Status Code :'.$e->getCode());
            	if($e instanceof ConnectException){
					$logger->warn("[{$e->getCode()}] {$e->getMessage()}");
                    $isTimedOut = true;
                }
                throw new ApiException(
                    "[{$e->getCode()}] {$e->getMessage()}",
                    $e->getCode(),
                    $e->getResponse() ? $e->getResponse()->getHeaders() : null,
                    $e->getResponse() ? $e->getResponse()->getBody()->getContents() : null, CommonUtil::get_masked_string(CommonUtil::wrapHttpRequest($request, $_tempBody, $formParams)), $masked_authentication_data
                ,$isTimedOut);
            }

            $statusCode = $response->getStatusCode();
			$logger->info('Receive Response from Service : HTTP Status Code :'.$statusCode);
            if ($statusCode < 200 || $statusCode > 299) {
                throw new ApiException(
                    sprintf(
                        '[%d] Error connecting to the API (%s)',
                        $statusCode,
                        $request->getUri()
                    ),
                    $statusCode,
                    $response->getHeaders(),
                    $response->getBody(), CommonUtil::get_masked_string(CommonUtil::wrapHttpRequest($request, $_tempBody, $formParams)), $masked_authentication_data
                );
            }

            $responseBody = $response->getBody();
            if ($returnType === '\SplFileObject') {
                $content = $responseBody; //stream goes to serializer
            } else {
                $content = $responseBody->getContents();
                if ($returnType !== 'string') {
                    $content = json_decode($content);
                }
            }

            return [
                ObjectSerializer::deserialize($content, $returnType, []),
                $response->getStatusCode(),
                $response->getHeaders()
            ];

        } catch (ApiException $e) {
            switch ($e->getCode()) {
                case 200:
                    $data = ObjectSerializer::deserialize(
                        json_decode($e->getResponseBody()),
                        '\Orbipay\PaymentsApi\Client\Model\DeleteResponse',
                        $e->getResponseHeaders()
                    );
                    $e->setResponseObject($data);
                    break;
                case 400:
                    $data = ObjectSerializer::deserialize(
                        json_decode($e->getResponseBody()),
                        '\Orbipay\PaymentsApi\Client\Model\ErrorResponseVo',
                        $e->getResponseHeaders()
                    );
                    $e->setResponseObject($data);
                    break;
                case 409:
                    $data = ObjectSerializer::deserialize(
                        json_decode($e->getResponseBody()),
                        '\Orbipay\PaymentsApi\Client\Model\ErrorResponseVo',
                        $e->getResponseHeaders()
                    );
                    $e->setResponseObject($data);
                    break;
                case 422:
                    $data = ObjectSerializer::deserialize(
                        json_decode($e->getResponseBody()),
                        '\Orbipay\PaymentsApi\Client\Model\ErrorResponseVo',
                        $e->getResponseHeaders()
                    );
                    $e->setResponseObject($data);
                    break;
            }
            throw $e;
        }
    }


    /**
     * Create request for operation 'deletePayment'
     *
     * @param  string $channel The channel through which the API is invoked. (required)
     * @param  string $client_key The unique identifier assigned by EBPP to the client. (required)
     * @param  string $product The product identifier corresponding to the API. (required)
     * @param  string $timestamp The timestamp for the moment when the API request is created. (required)
     * @param  string $idempotent_request_key The unique token that clients can generate and maintain in order to identify an API request. (required)
     * @param  string $requestor_type Type of the requestor of the API. (required)
     * @param  string $id_payment The unique identifier assigned by EBPP to the payment. (required)
     * @param  string $id_customer The unique identifier assigned by EBPP to the customer. (required)
     * @param  DeletePaymentRequest $delete_payment_request Details of payment to be deleted/cancelled (required)
     * @param  string $requestor The identifier for the requestor of the API. (optional)
     * @param  string $x_opay_headers Intended for the future use. (optional)
     * @param  string $trace_id The unique reference that can be used for tracing and debugging an API call. (optional)
     *
     * @throws \InvalidArgumentException
     * @return array
     */
    protected function deletePaymentRequest($channel, $client_key, $product, $timestamp, $idempotent_request_key, $requestor_type, $id_payment, $id_customer, $delete_payment_request, $requestor = null, $x_opay_headers = null, $trace_id = null)
    {
    
     $logger = LoggerFactory::getLogger(__CLASS__);
    
        // verify the required parameter 'id_payment' is set
        if ($id_payment === null || (is_array($id_payment) && count($id_payment) === 0) || strlen($id_payment) === 0) {
     		$logger->error('Missing the required parameter $id_payment');
           throw new \InvalidArgumentException(
                 'Missing the required parameter $id_payment'
             );
        }
        // verify the required parameter 'id_customer' is set
        if ($id_customer === null || (is_array($id_customer) && count($id_customer) === 0) || strlen($id_customer) === 0) {
     		$logger->error('Missing the required parameter $id_customer');
           throw new \InvalidArgumentException(
                 'Missing the required parameter $id_customer'
             );
        }
       
		$method = 'DELETE';
        $resourcePath = '/customers/{ID_CUSTOMER}/payments/{ID_PAYMENT}';
        $formParams = [];
        $queryParams = [];
        $headerParams = [];
        $httpBody = '';
        $multipart = false;
        $masked_authentication_data = '';

        // header params
        if ($channel !== null) {
            $headerParams['channel'] = ObjectSerializer::toHeaderValue($channel);
        }
        // header params
        if ($client_key !== null) {
            $headerParams['client_key'] = ObjectSerializer::toHeaderValue($client_key);
        }
        // header params
        if ($product !== null) {
            $headerParams['product'] = ObjectSerializer::toHeaderValue($product);
        }
        // header params
        if ($timestamp !== null) {
            $headerParams['timestamp'] = ObjectSerializer::toHeaderValue($timestamp);
        }
        // header params
        if ($idempotent_request_key !== null) {
            $headerParams['idempotent_request_key'] = ObjectSerializer::toHeaderValue($idempotent_request_key);
        }
        // header params
        if ($requestor_type !== null) {
            $headerParams['requestor_type'] = ObjectSerializer::toHeaderValue($requestor_type);
        }
        // header params
        if ($requestor !== null) {
            $headerParams['requestor'] = ObjectSerializer::toHeaderValue($requestor);
        }
        // header params
        if ($x_opay_headers !== null) {
            $headerParams['X-OPAY-Headers'] = ObjectSerializer::toHeaderValue($x_opay_headers);
        }
        // header params
        if ($trace_id !== null) {
            $headerParams['trace_id'] = ObjectSerializer::toHeaderValue($trace_id);
        }

        // path params
        if ($id_payment !== null) {
            $resourcePath = str_replace(
                '{' . 'ID_PAYMENT' . '}',
                ObjectSerializer::toPathValue($id_payment),
                $resourcePath
            );
        }
        // path params
        if ($id_customer !== null) {
            $resourcePath = str_replace(
                '{' . 'ID_CUSTOMER' . '}',
                ObjectSerializer::toPathValue($id_customer),
                $resourcePath
            );
        }

        // body params
        $_tempBody = null;
        if (isset($delete_payment_request)) {
            $_tempBody = $delete_payment_request;
        }

        if ($multipart) {
            $headers = $this->headerSelector->selectHeadersForMultipart(
                ['application/json']
            );
        } else {
            $headers = $this->headerSelector->selectHeaders(
                ['application/json'],
                ['application/json']
            );
        }

        // for model (json/xml)
        if (isset($_tempBody)) {
            // $_tempBody is the method argument, if present
            $httpBody = $_tempBody;
            // \stdClass has no __toString(), so we should encode it manually
            if ($httpBody instanceof \stdClass && $headers['Content-Type'] === 'application/json') {
                $httpBody = \GuzzleHttp\json_encode($httpBody);
            }
        } elseif (count($formParams) > 0) {
            if ($multipart) {
                $multipartContents = [];
                foreach ($formParams as $formParamName => $formParamValue) {
                    $multipartContents[] = [
                        'name' => $formParamName,
                        'contents' => $formParamValue
                    ];
                }
                // for HTTP post (form)
                $httpBody = new MultipartStream($multipartContents);

            } elseif ($headers['Content-Type'] === 'application/json') {
                $httpBody = \GuzzleHttp\json_encode($formParams);

            } else {
                // for HTTP post (form)
                $httpBody = \GuzzleHttp\Psr7\build_query($formParams);
            }
        }
        
        if($logger->isDebugEnabled()){
            $logger->debug('HTTP Method     :'.$method);
            $logger->debug('HTTP URL        :'.$this->config->getHost() . $resourcePath);
        }
        
         if(empty($this->getConfig()->getClientApiKey())){
        	$this->getConfig()->setClientApiKey($this->getConfig()->getClientKey());
        }
        
        if(empty($this->getConfig()->getAuthScheme()) || $this->getConfig()->authScheme1 == $this->getConfig()->getAuthScheme()){
	         
	        list($hash, $masked_authentication_data) = AuthenticationUtil::computeOPAY1HMACSHA256Hash($method, $resourcePath, $headerParams, $queryParams, $formParams, $_tempBody, $this->getConfig()->getClientSignatureKey());
	        $headerParams['Authorization'] = $this->getConfig()->authScheme1 . " Credential=" . $this->getConfig()->getClientApiKey() . ",Signature=" . $hash;
        
        }elseif($this->getConfig()->authScheme2 == $this->getConfig()->getAuthScheme()){
				
	        $hash = AuthenticationUtil::computeOPAY2HMACSHA256Hash($this->getConfig()->getClientKey(), $this->getConfig()->getClientApiKey() ,$this->getConfig()->getClientSignatureKey(), $headerParams['timestamp']);
	        $headerParams['Authorization'] = $this->getConfig()->authScheme2 . " Credential=" . $this->getConfig()->getClientApiKey() . ",Signature=" . $hash;
	                
        }elseif($this->getConfig()->authScheme3 == $this->getConfig()->getAuthScheme()){
        	$headerParams['Authorization'] = $this->getConfig()->authScheme3 . " Credential=" . $this->getConfig()->getClientApiKey();
        }else{
        	$logger->error('Invalid auth_scheme received.');
           throw new \InvalidArgumentException('Invalid auth_scheme received.');
        }
        


        $defaultHeaders = [];
        if ($this->config->getUserAgent()) {
            $defaultHeaders['User-Agent'] = $this->config->getUserAgent();
        }

        $headers = array_merge(
            $defaultHeaders,
            $headerParams,
            $headers
        );

		if($logger->isDebugEnabled()){
			$logger->debug('Request Headers  :'.json_encode($headers));
		}
		
        $query = \GuzzleHttp\Psr7\build_query($queryParams);
        return [ new Request(
            $method,
            $this->config->getHost() . $resourcePath . ($query ? "?{$query}" : ''),
            $headers,
            $httpBody
        ), $_tempBody, $formParams, $masked_authentication_data];
    }

    /**
     * Operation deleteRecordPayment
     *
     * Delete Recorded Payment
     *
     * @param  string $channel The channel through which the API is invoked. (required)
     * @param  string $client_key The unique identifier assigned by EBPP to the client. (required)
     * @param  string $product The product identifier corresponding to the API. (required)
     * @param  string $timestamp The timestamp for the moment when the API request is created. (required)
     * @param  string $idempotent_request_key The unique token that clients can generate and maintain in order to identify an API request. (required)
     * @param  string $requestor_type Type of the requestor of the API. (required)
     * @param  string $id_payment The unique identifier assigned by EBPP to the payment. (required)
     * @param  DeleteRecordedPaymentRequest $delete_recorded_payment_request Details of payment to be deleted/cancelled (required)
     * @param  string $requestor The identifier for the requestor of the API. (optional)
     * @param  string $x_opay_headers Intended for the future use. (optional)
     * @param  string $trace_id The unique reference that can be used for tracing and debugging an API call. (optional)
     *
     * @throws \Orbipay\PaymentsApi\Client\ApiException on non-2xx response
     * @throws \InvalidArgumentException
     * @return array of void, HTTP status code, HTTP response headers (array of strings)
     */
    public function deleteRecordPayment($channel, $client_key, $product, $timestamp, $idempotent_request_key, $requestor_type, $id_payment, $delete_recorded_payment_request, $requestor = null, $x_opay_headers = null, $trace_id = null)
    {
        $response = $this->deleteRecordPaymentWithHttpInfo($channel, $client_key, $product, $timestamp, $idempotent_request_key, $requestor_type, $id_payment, $delete_recorded_payment_request, $requestor, $x_opay_headers, $trace_id);
        return $response;
    }

    /**
     * Operation deleteRecordPaymentWithHttpInfo
     *
     * Delete Recorded Payment
     *
     * @param  string $channel The channel through which the API is invoked. (required)
     * @param  string $client_key The unique identifier assigned by EBPP to the client. (required)
     * @param  string $product The product identifier corresponding to the API. (required)
     * @param  string $timestamp The timestamp for the moment when the API request is created. (required)
     * @param  string $idempotent_request_key The unique token that clients can generate and maintain in order to identify an API request. (required)
     * @param  string $requestor_type Type of the requestor of the API. (required)
     * @param  string $id_payment The unique identifier assigned by EBPP to the payment. (required)
     * @param  DeleteRecordedPaymentRequest $delete_recorded_payment_request Details of payment to be deleted/cancelled (required)
     * @param  string $requestor The identifier for the requestor of the API. (optional)
     * @param  string $x_opay_headers Intended for the future use. (optional)
     * @param  string $trace_id The unique reference that can be used for tracing and debugging an API call. (optional)
     *
     * @throws \Orbipay\PaymentsApi\Client\ApiException on non-2xx response
     * @throws \InvalidArgumentException
     * @return array of null, HTTP status code, HTTP response headers (array of strings)
     */
    public function deleteRecordPaymentWithHttpInfo($channel, $client_key, $product, $timestamp, $idempotent_request_key, $requestor_type, $id_payment, $delete_recorded_payment_request, $requestor = null, $x_opay_headers = null, $trace_id = null)
    {
        
        list($request, $_tempBody, $formParams, $masked_authentication_data) = $this->deleteRecordPaymentRequest($channel, $client_key, $product, $timestamp, $idempotent_request_key, $requestor_type, $id_payment, $delete_recorded_payment_request, $requestor, $x_opay_headers, $trace_id);
		$logger = LoggerFactory::getLogger(__CLASS__);
        try {
            $options = $this->createHttpClientOption();
            try {
             	$isTimedOut = false;
             	if($logger->isDebugEnabled()){
             		$logger->debug('Connect TimeOut :'.$options[RequestOptions::TIMEOUT]);
             		$logger->debug('Read TimeOut :'.$options[RequestOptions::TIMEOUT]);
             	}
             	$logger->info('Sending Request to Service');
                $response = $this->client->send($request, $options);
            } catch (RequestException $e) {
            	$logger->info('Receive Response from Service : HTTP Status Code :'.$e->getCode());
            	if($e instanceof ConnectException){
					$logger->warn("[{$e->getCode()}] {$e->getMessage()}");
                    $isTimedOut = true;
                }
                throw new ApiException(
                    "[{$e->getCode()}] {$e->getMessage()}",
                    $e->getCode(),
                    $e->getResponse() ? $e->getResponse()->getHeaders() : null,
                    $e->getResponse() ? $e->getResponse()->getBody()->getContents() : null, CommonUtil::get_masked_string(CommonUtil::wrapHttpRequest($request, $_tempBody, $formParams)), $masked_authentication_data
                ,$isTimedOut);
            }

            $statusCode = $response->getStatusCode();
			$logger->info('Receive Response from Service : HTTP Status Code :'.$statusCode);
            if ($statusCode < 200 || $statusCode > 299) {
                throw new ApiException(
                    sprintf(
                        '[%d] Error connecting to the API (%s)',
                        $statusCode,
                        $request->getUri()
                    ),
                    $statusCode,
                    $response->getHeaders(),
                    $response->getBody(), CommonUtil::get_masked_string(CommonUtil::wrapHttpRequest($request, $_tempBody, $formParams)), $masked_authentication_data
                );
            }

            return [null, $statusCode, $response->getHeaders()];

        } catch (ApiException $e) {
            switch ($e->getCode()) {
                case 400:
                    $data = ObjectSerializer::deserialize(
                        json_decode($e->getResponseBody()),
                        '\Orbipay\PaymentsApi\Client\Model\ErrorResponseVo',
                        $e->getResponseHeaders()
                    );
                    $e->setResponseObject($data);
                    break;
                case 409:
                    $data = ObjectSerializer::deserialize(
                        json_decode($e->getResponseBody()),
                        '\Orbipay\PaymentsApi\Client\Model\ErrorResponseVo',
                        $e->getResponseHeaders()
                    );
                    $e->setResponseObject($data);
                    break;
                case 422:
                    $data = ObjectSerializer::deserialize(
                        json_decode($e->getResponseBody()),
                        '\Orbipay\PaymentsApi\Client\Model\ErrorResponseVo',
                        $e->getResponseHeaders()
                    );
                    $e->setResponseObject($data);
                    break;
            }
            throw $e;
        }
    }


    /**
     * Create request for operation 'deleteRecordPayment'
     *
     * @param  string $channel The channel through which the API is invoked. (required)
     * @param  string $client_key The unique identifier assigned by EBPP to the client. (required)
     * @param  string $product The product identifier corresponding to the API. (required)
     * @param  string $timestamp The timestamp for the moment when the API request is created. (required)
     * @param  string $idempotent_request_key The unique token that clients can generate and maintain in order to identify an API request. (required)
     * @param  string $requestor_type Type of the requestor of the API. (required)
     * @param  string $id_payment The unique identifier assigned by EBPP to the payment. (required)
     * @param  DeleteRecordedPaymentRequest $delete_recorded_payment_request Details of payment to be deleted/cancelled (required)
     * @param  string $requestor The identifier for the requestor of the API. (optional)
     * @param  string $x_opay_headers Intended for the future use. (optional)
     * @param  string $trace_id The unique reference that can be used for tracing and debugging an API call. (optional)
     *
     * @throws \InvalidArgumentException
     * @return array
     */
    protected function deleteRecordPaymentRequest($channel, $client_key, $product, $timestamp, $idempotent_request_key, $requestor_type, $id_payment, $delete_recorded_payment_request, $requestor = null, $x_opay_headers = null, $trace_id = null)
    {
    
     $logger = LoggerFactory::getLogger(__CLASS__);
    
        // verify the required parameter 'id_payment' is set
        if ($id_payment === null || (is_array($id_payment) && count($id_payment) === 0) || strlen($id_payment) === 0) {
     		$logger->error('Missing the required parameter $id_payment');
           throw new \InvalidArgumentException(
                 'Missing the required parameter $id_payment'
             );
        }
       
		$method = 'DELETE';
        $resourcePath = '/recordedpayments/{ID_PAYMENT}';
        $formParams = [];
        $queryParams = [];
        $headerParams = [];
        $httpBody = '';
        $multipart = false;
        $masked_authentication_data = '';

        // header params
        if ($channel !== null) {
            $headerParams['channel'] = ObjectSerializer::toHeaderValue($channel);
        }
        // header params
        if ($client_key !== null) {
            $headerParams['client_key'] = ObjectSerializer::toHeaderValue($client_key);
        }
        // header params
        if ($product !== null) {
            $headerParams['product'] = ObjectSerializer::toHeaderValue($product);
        }
        // header params
        if ($timestamp !== null) {
            $headerParams['timestamp'] = ObjectSerializer::toHeaderValue($timestamp);
        }
        // header params
        if ($idempotent_request_key !== null) {
            $headerParams['idempotent_request_key'] = ObjectSerializer::toHeaderValue($idempotent_request_key);
        }
        // header params
        if ($requestor_type !== null) {
            $headerParams['requestor_type'] = ObjectSerializer::toHeaderValue($requestor_type);
        }
        // header params
        if ($requestor !== null) {
            $headerParams['requestor'] = ObjectSerializer::toHeaderValue($requestor);
        }
        // header params
        if ($x_opay_headers !== null) {
            $headerParams['X-OPAY-Headers'] = ObjectSerializer::toHeaderValue($x_opay_headers);
        }
        // header params
        if ($trace_id !== null) {
            $headerParams['trace_id'] = ObjectSerializer::toHeaderValue($trace_id);
        }

        // path params
        if ($id_payment !== null) {
            $resourcePath = str_replace(
                '{' . 'ID_PAYMENT' . '}',
                ObjectSerializer::toPathValue($id_payment),
                $resourcePath
            );
        }

        // body params
        $_tempBody = null;
        if (isset($delete_recorded_payment_request)) {
            $_tempBody = $delete_recorded_payment_request;
        }

        if ($multipart) {
            $headers = $this->headerSelector->selectHeadersForMultipart(
                ['application/json']
            );
        } else {
            $headers = $this->headerSelector->selectHeaders(
                ['application/json'],
                ['application/json']
            );
        }

        // for model (json/xml)
        if (isset($_tempBody)) {
            // $_tempBody is the method argument, if present
            $httpBody = $_tempBody;
            // \stdClass has no __toString(), so we should encode it manually
            if ($httpBody instanceof \stdClass && $headers['Content-Type'] === 'application/json') {
                $httpBody = \GuzzleHttp\json_encode($httpBody);
            }
        } elseif (count($formParams) > 0) {
            if ($multipart) {
                $multipartContents = [];
                foreach ($formParams as $formParamName => $formParamValue) {
                    $multipartContents[] = [
                        'name' => $formParamName,
                        'contents' => $formParamValue
                    ];
                }
                // for HTTP post (form)
                $httpBody = new MultipartStream($multipartContents);

            } elseif ($headers['Content-Type'] === 'application/json') {
                $httpBody = \GuzzleHttp\json_encode($formParams);

            } else {
                // for HTTP post (form)
                $httpBody = \GuzzleHttp\Psr7\build_query($formParams);
            }
        }
        
        if($logger->isDebugEnabled()){
            $logger->debug('HTTP Method     :'.$method);
            $logger->debug('HTTP URL        :'.$this->config->getHost() . $resourcePath);
        }
        
         if(empty($this->getConfig()->getClientApiKey())){
        	$this->getConfig()->setClientApiKey($this->getConfig()->getClientKey());
        }
        
        if(empty($this->getConfig()->getAuthScheme()) || $this->getConfig()->authScheme1 == $this->getConfig()->getAuthScheme()){
	         
	        list($hash, $masked_authentication_data) = AuthenticationUtil::computeOPAY1HMACSHA256Hash($method, $resourcePath, $headerParams, $queryParams, $formParams, $_tempBody, $this->getConfig()->getClientSignatureKey());
	        $headerParams['Authorization'] = $this->getConfig()->authScheme1 . " Credential=" . $this->getConfig()->getClientApiKey() . ",Signature=" . $hash;
        
        }elseif($this->getConfig()->authScheme2 == $this->getConfig()->getAuthScheme()){
				
	        $hash = AuthenticationUtil::computeOPAY2HMACSHA256Hash($this->getConfig()->getClientKey(), $this->getConfig()->getClientApiKey() ,$this->getConfig()->getClientSignatureKey(), $headerParams['timestamp']);
	        $headerParams['Authorization'] = $this->getConfig()->authScheme2 . " Credential=" . $this->getConfig()->getClientApiKey() . ",Signature=" . $hash;
	                
        }elseif($this->getConfig()->authScheme3 == $this->getConfig()->getAuthScheme()){
        	$headerParams['Authorization'] = $this->getConfig()->authScheme3 . " Credential=" . $this->getConfig()->getClientApiKey();
        }else{
        	$logger->error('Invalid auth_scheme received.');
           throw new \InvalidArgumentException('Invalid auth_scheme received.');
        }
        


        $defaultHeaders = [];
        if ($this->config->getUserAgent()) {
            $defaultHeaders['User-Agent'] = $this->config->getUserAgent();
        }

        $headers = array_merge(
            $defaultHeaders,
            $headerParams,
            $headers
        );

		if($logger->isDebugEnabled()){
			$logger->debug('Request Headers  :'.json_encode($headers));
		}
		
        $query = \GuzzleHttp\Psr7\build_query($queryParams);
        return [ new Request(
            $method,
            $this->config->getHost() . $resourcePath . ($query ? "?{$query}" : ''),
            $headers,
            $httpBody
        ), $_tempBody, $formParams, $masked_authentication_data];
    }

    /**
     * Operation getPayment
     *
     * Get Payment
     *
     * @param  string $channel The channel through which the API is invoked. (required)
     * @param  string $client_key The unique identifier assigned by EBPP to the client. (required)
     * @param  string $product The product identifier corresponding to the API. (required)
     * @param  string $timestamp The timestamp for the moment when the API request is created. (required)
     * @param  string $idempotent_request_key The unique token that clients can generate and maintain in order to identify an API request. (required)
     * @param  string $requestor_type Type of the requestor of the API. (required)
     * @param  string $id_payment The unique identifier assigned by EBPP to the payment. (required)
     * @param  string $id_customer The unique identifier assigned by EBPP to the customer. (required)
     * @param  string $requestor The identifier for the requestor of the API. (optional)
     * @param  string $x_opay_headers Intended for the future use. (optional)
     * @param  string $trace_id The unique reference that can be used for tracing and debugging an API call. (optional)
     *
     * @throws \Orbipay\PaymentsApi\Client\ApiException on non-2xx response
     * @throws \InvalidArgumentException
     * @return array of \Orbipay\PaymentsApi\Client\Model\PaymentResponse, HTTP status code, HTTP response headers (array of strings)
     */
    public function getPayment($channel, $client_key, $product, $timestamp, $idempotent_request_key, $requestor_type, $id_payment, $id_customer, $requestor = null, $x_opay_headers = null, $trace_id = null)
    {
        $response = $this->getPaymentWithHttpInfo($channel, $client_key, $product, $timestamp, $idempotent_request_key, $requestor_type, $id_payment, $id_customer, $requestor, $x_opay_headers, $trace_id);
        return $response;
    }

    /**
     * Operation getPaymentWithHttpInfo
     *
     * Get Payment
     *
     * @param  string $channel The channel through which the API is invoked. (required)
     * @param  string $client_key The unique identifier assigned by EBPP to the client. (required)
     * @param  string $product The product identifier corresponding to the API. (required)
     * @param  string $timestamp The timestamp for the moment when the API request is created. (required)
     * @param  string $idempotent_request_key The unique token that clients can generate and maintain in order to identify an API request. (required)
     * @param  string $requestor_type Type of the requestor of the API. (required)
     * @param  string $id_payment The unique identifier assigned by EBPP to the payment. (required)
     * @param  string $id_customer The unique identifier assigned by EBPP to the customer. (required)
     * @param  string $requestor The identifier for the requestor of the API. (optional)
     * @param  string $x_opay_headers Intended for the future use. (optional)
     * @param  string $trace_id The unique reference that can be used for tracing and debugging an API call. (optional)
     *
     * @throws \Orbipay\PaymentsApi\Client\ApiException on non-2xx response
     * @throws \InvalidArgumentException
     * @return array of \Orbipay\PaymentsApi\Client\Model\PaymentResponse, HTTP status code, HTTP response headers (array of strings)
     */
    public function getPaymentWithHttpInfo($channel, $client_key, $product, $timestamp, $idempotent_request_key, $requestor_type, $id_payment, $id_customer, $requestor = null, $x_opay_headers = null, $trace_id = null)
    {
        $returnType = '\Orbipay\PaymentsApi\Client\Model\PaymentResponse';
        list($request, $_tempBody, $formParams, $masked_authentication_data) = $this->getPaymentRequest($channel, $client_key, $product, $timestamp, $idempotent_request_key, $requestor_type, $id_payment, $id_customer, $requestor, $x_opay_headers, $trace_id);
		$logger = LoggerFactory::getLogger(__CLASS__);
        try {
            $options = $this->createHttpClientOption();
            try {
             	$isTimedOut = false;
             	if($logger->isDebugEnabled()){
             		$logger->debug('Connect TimeOut :'.$options[RequestOptions::TIMEOUT]);
             		$logger->debug('Read TimeOut :'.$options[RequestOptions::TIMEOUT]);
             	}
             	$logger->info('Sending Request to Service');
                $response = $this->client->send($request, $options);
            } catch (RequestException $e) {
            	$logger->info('Receive Response from Service : HTTP Status Code :'.$e->getCode());
            	if($e instanceof ConnectException){
					$logger->warn("[{$e->getCode()}] {$e->getMessage()}");
                    $isTimedOut = true;
                }
                throw new ApiException(
                    "[{$e->getCode()}] {$e->getMessage()}",
                    $e->getCode(),
                    $e->getResponse() ? $e->getResponse()->getHeaders() : null,
                    $e->getResponse() ? $e->getResponse()->getBody()->getContents() : null, CommonUtil::get_masked_string(CommonUtil::wrapHttpRequest($request, $_tempBody, $formParams)), $masked_authentication_data
                ,$isTimedOut);
            }

            $statusCode = $response->getStatusCode();
			$logger->info('Receive Response from Service : HTTP Status Code :'.$statusCode);
            if ($statusCode < 200 || $statusCode > 299) {
                throw new ApiException(
                    sprintf(
                        '[%d] Error connecting to the API (%s)',
                        $statusCode,
                        $request->getUri()
                    ),
                    $statusCode,
                    $response->getHeaders(),
                    $response->getBody(), CommonUtil::get_masked_string(CommonUtil::wrapHttpRequest($request, $_tempBody, $formParams)), $masked_authentication_data
                );
            }

            $responseBody = $response->getBody();
            if ($returnType === '\SplFileObject') {
                $content = $responseBody; //stream goes to serializer
            } else {
                $content = $responseBody->getContents();
                if ($returnType !== 'string') {
                    $content = json_decode($content);
                }
            }

            return [
                ObjectSerializer::deserialize($content, $returnType, []),
                $response->getStatusCode(),
                $response->getHeaders()
            ];

        } catch (ApiException $e) {
            switch ($e->getCode()) {
                case 200:
                    $data = ObjectSerializer::deserialize(
                        json_decode($e->getResponseBody()),
                        '\Orbipay\PaymentsApi\Client\Model\PaymentResponse',
                        $e->getResponseHeaders()
                    );
                    $e->setResponseObject($data);
                    break;
                case 400:
                    $data = ObjectSerializer::deserialize(
                        json_decode($e->getResponseBody()),
                        '\Orbipay\PaymentsApi\Client\Model\ErrorResponseVo',
                        $e->getResponseHeaders()
                    );
                    $e->setResponseObject($data);
                    break;
                case 409:
                    $data = ObjectSerializer::deserialize(
                        json_decode($e->getResponseBody()),
                        '\Orbipay\PaymentsApi\Client\Model\ErrorResponseVo',
                        $e->getResponseHeaders()
                    );
                    $e->setResponseObject($data);
                    break;
                case 422:
                    $data = ObjectSerializer::deserialize(
                        json_decode($e->getResponseBody()),
                        '\Orbipay\PaymentsApi\Client\Model\ErrorResponseVo',
                        $e->getResponseHeaders()
                    );
                    $e->setResponseObject($data);
                    break;
            }
            throw $e;
        }
    }


    /**
     * Create request for operation 'getPayment'
     *
     * @param  string $channel The channel through which the API is invoked. (required)
     * @param  string $client_key The unique identifier assigned by EBPP to the client. (required)
     * @param  string $product The product identifier corresponding to the API. (required)
     * @param  string $timestamp The timestamp for the moment when the API request is created. (required)
     * @param  string $idempotent_request_key The unique token that clients can generate and maintain in order to identify an API request. (required)
     * @param  string $requestor_type Type of the requestor of the API. (required)
     * @param  string $id_payment The unique identifier assigned by EBPP to the payment. (required)
     * @param  string $id_customer The unique identifier assigned by EBPP to the customer. (required)
     * @param  string $requestor The identifier for the requestor of the API. (optional)
     * @param  string $x_opay_headers Intended for the future use. (optional)
     * @param  string $trace_id The unique reference that can be used for tracing and debugging an API call. (optional)
     *
     * @throws \InvalidArgumentException
     * @return array
     */
    protected function getPaymentRequest($channel, $client_key, $product, $timestamp, $idempotent_request_key, $requestor_type, $id_payment, $id_customer, $requestor = null, $x_opay_headers = null, $trace_id = null)
    {
    
     $logger = LoggerFactory::getLogger(__CLASS__);
    
        // verify the required parameter 'id_payment' is set
        if ($id_payment === null || (is_array($id_payment) && count($id_payment) === 0) || strlen($id_payment) === 0) {
     		$logger->error('Missing the required parameter $id_payment');
           throw new \InvalidArgumentException(
                 'Missing the required parameter $id_payment'
             );
        }
        // verify the required parameter 'id_customer' is set
        if ($id_customer === null || (is_array($id_customer) && count($id_customer) === 0) || strlen($id_customer) === 0) {
     		$logger->error('Missing the required parameter $id_customer');
           throw new \InvalidArgumentException(
                 'Missing the required parameter $id_customer'
             );
        }
       
		$method = 'GET';
        $resourcePath = '/customers/{ID_CUSTOMER}/payments/{ID_PAYMENT}';
        $formParams = [];
        $queryParams = [];
        $headerParams = [];
        $httpBody = '';
        $multipart = false;
        $masked_authentication_data = '';

        // header params
        if ($channel !== null) {
            $headerParams['channel'] = ObjectSerializer::toHeaderValue($channel);
        }
        // header params
        if ($client_key !== null) {
            $headerParams['client_key'] = ObjectSerializer::toHeaderValue($client_key);
        }
        // header params
        if ($product !== null) {
            $headerParams['product'] = ObjectSerializer::toHeaderValue($product);
        }
        // header params
        if ($timestamp !== null) {
            $headerParams['timestamp'] = ObjectSerializer::toHeaderValue($timestamp);
        }
        // header params
        if ($idempotent_request_key !== null) {
            $headerParams['idempotent_request_key'] = ObjectSerializer::toHeaderValue($idempotent_request_key);
        }
        // header params
        if ($requestor_type !== null) {
            $headerParams['requestor_type'] = ObjectSerializer::toHeaderValue($requestor_type);
        }
        // header params
        if ($requestor !== null) {
            $headerParams['requestor'] = ObjectSerializer::toHeaderValue($requestor);
        }
        // header params
        if ($x_opay_headers !== null) {
            $headerParams['X-OPAY-Headers'] = ObjectSerializer::toHeaderValue($x_opay_headers);
        }
        // header params
        if ($trace_id !== null) {
            $headerParams['trace_id'] = ObjectSerializer::toHeaderValue($trace_id);
        }

        // path params
        if ($id_payment !== null) {
            $resourcePath = str_replace(
                '{' . 'ID_PAYMENT' . '}',
                ObjectSerializer::toPathValue($id_payment),
                $resourcePath
            );
        }
        // path params
        if ($id_customer !== null) {
            $resourcePath = str_replace(
                '{' . 'ID_CUSTOMER' . '}',
                ObjectSerializer::toPathValue($id_customer),
                $resourcePath
            );
        }

        // body params
        $_tempBody = null;

        if ($multipart) {
            $headers = $this->headerSelector->selectHeadersForMultipart(
                ['application/json']
            );
        } else {
            $headers = $this->headerSelector->selectHeaders(
                ['application/json'],
                []
            );
        }

        // for model (json/xml)
        if (isset($_tempBody)) {
            // $_tempBody is the method argument, if present
            $httpBody = $_tempBody;
            // \stdClass has no __toString(), so we should encode it manually
            if ($httpBody instanceof \stdClass && $headers['Content-Type'] === 'application/json') {
                $httpBody = \GuzzleHttp\json_encode($httpBody);
            }
        } elseif (count($formParams) > 0) {
            if ($multipart) {
                $multipartContents = [];
                foreach ($formParams as $formParamName => $formParamValue) {
                    $multipartContents[] = [
                        'name' => $formParamName,
                        'contents' => $formParamValue
                    ];
                }
                // for HTTP post (form)
                $httpBody = new MultipartStream($multipartContents);

            } elseif ($headers['Content-Type'] === 'application/json') {
                $httpBody = \GuzzleHttp\json_encode($formParams);

            } else {
                // for HTTP post (form)
                $httpBody = \GuzzleHttp\Psr7\build_query($formParams);
            }
        }
        
        if($logger->isDebugEnabled()){
            $logger->debug('HTTP Method     :'.$method);
            $logger->debug('HTTP URL        :'.$this->config->getHost() . $resourcePath);
        }
        
         if(empty($this->getConfig()->getClientApiKey())){
        	$this->getConfig()->setClientApiKey($this->getConfig()->getClientKey());
        }
        
        if(empty($this->getConfig()->getAuthScheme()) || $this->getConfig()->authScheme1 == $this->getConfig()->getAuthScheme()){
	         
	        list($hash, $masked_authentication_data) = AuthenticationUtil::computeOPAY1HMACSHA256Hash($method, $resourcePath, $headerParams, $queryParams, $formParams, $_tempBody, $this->getConfig()->getClientSignatureKey());
	        $headerParams['Authorization'] = $this->getConfig()->authScheme1 . " Credential=" . $this->getConfig()->getClientApiKey() . ",Signature=" . $hash;
        
        }elseif($this->getConfig()->authScheme2 == $this->getConfig()->getAuthScheme()){
				
	        $hash = AuthenticationUtil::computeOPAY2HMACSHA256Hash($this->getConfig()->getClientKey(), $this->getConfig()->getClientApiKey() ,$this->getConfig()->getClientSignatureKey(), $headerParams['timestamp']);
	        $headerParams['Authorization'] = $this->getConfig()->authScheme2 . " Credential=" . $this->getConfig()->getClientApiKey() . ",Signature=" . $hash;
	                
        }elseif($this->getConfig()->authScheme3 == $this->getConfig()->getAuthScheme()){
        	$headerParams['Authorization'] = $this->getConfig()->authScheme3 . " Credential=" . $this->getConfig()->getClientApiKey();
        }else{
        	$logger->error('Invalid auth_scheme received.');
           throw new \InvalidArgumentException('Invalid auth_scheme received.');
        }
        


        $defaultHeaders = [];
        if ($this->config->getUserAgent()) {
            $defaultHeaders['User-Agent'] = $this->config->getUserAgent();
        }

        $headers = array_merge(
            $defaultHeaders,
            $headerParams,
            $headers
        );

		if($logger->isDebugEnabled()){
			$logger->debug('Request Headers  :'.json_encode($headers));
		}
		
        $query = \GuzzleHttp\Psr7\build_query($queryParams);
        return [ new Request(
            $method,
            $this->config->getHost() . $resourcePath . ($query ? "?{$query}" : ''),
            $headers,
            $httpBody
        ), $_tempBody, $formParams, $masked_authentication_data];
    }

    /**
     * Operation recordPayment
     *
     * Record Payment
     *
     * @param  string $channel The channel through which the API is invoked. (required)
     * @param  string $client_key The unique identifier assigned by EBPP to the client. (required)
     * @param  string $product The product identifier corresponding to the API. (required)
     * @param  string $timestamp The timestamp for the moment when the API request is created. (required)
     * @param  string $idempotent_request_key The unique token that clients can generate and maintain in order to identify an API request. (required)
     * @param  string $requestor_type Type of the requestor of the API. (required)
     * @param  RecordPaymentRequest $record_payment_request JSON containing all the fields required to record a payment. (required)
     * @param  string $requestor The identifier for the requestor of the API. (optional)
     * @param  string $x_opay_headers Intended for the future use. (optional)
     * @param  string $trace_id The unique reference that can be used for tracing and debugging an API call. (optional)
     *
     * @throws \Orbipay\PaymentsApi\Client\ApiException on non-2xx response
     * @throws \InvalidArgumentException
     * @return array of \Orbipay\PaymentsApi\Client\Model\RecordedPaymentResponse, HTTP status code, HTTP response headers (array of strings)
     */
    public function recordPayment($channel, $client_key, $product, $timestamp, $idempotent_request_key, $requestor_type, $record_payment_request, $requestor = null, $x_opay_headers = null, $trace_id = null)
    {
        $response = $this->recordPaymentWithHttpInfo($channel, $client_key, $product, $timestamp, $idempotent_request_key, $requestor_type, $record_payment_request, $requestor, $x_opay_headers, $trace_id);
        return $response;
    }

    /**
     * Operation recordPaymentWithHttpInfo
     *
     * Record Payment
     *
     * @param  string $channel The channel through which the API is invoked. (required)
     * @param  string $client_key The unique identifier assigned by EBPP to the client. (required)
     * @param  string $product The product identifier corresponding to the API. (required)
     * @param  string $timestamp The timestamp for the moment when the API request is created. (required)
     * @param  string $idempotent_request_key The unique token that clients can generate and maintain in order to identify an API request. (required)
     * @param  string $requestor_type Type of the requestor of the API. (required)
     * @param  RecordPaymentRequest $record_payment_request JSON containing all the fields required to record a payment. (required)
     * @param  string $requestor The identifier for the requestor of the API. (optional)
     * @param  string $x_opay_headers Intended for the future use. (optional)
     * @param  string $trace_id The unique reference that can be used for tracing and debugging an API call. (optional)
     *
     * @throws \Orbipay\PaymentsApi\Client\ApiException on non-2xx response
     * @throws \InvalidArgumentException
     * @return array of \Orbipay\PaymentsApi\Client\Model\RecordedPaymentResponse, HTTP status code, HTTP response headers (array of strings)
     */
    public function recordPaymentWithHttpInfo($channel, $client_key, $product, $timestamp, $idempotent_request_key, $requestor_type, $record_payment_request, $requestor = null, $x_opay_headers = null, $trace_id = null)
    {
        $returnType = '\Orbipay\PaymentsApi\Client\Model\RecordedPaymentResponse';
        list($request, $_tempBody, $formParams, $masked_authentication_data) = $this->recordPaymentRequest($channel, $client_key, $product, $timestamp, $idempotent_request_key, $requestor_type, $record_payment_request, $requestor, $x_opay_headers, $trace_id);
		$logger = LoggerFactory::getLogger(__CLASS__);
        try {
            $options = $this->createHttpClientOption();
            try {
             	$isTimedOut = false;
             	if($logger->isDebugEnabled()){
             		$logger->debug('Connect TimeOut :'.$options[RequestOptions::TIMEOUT]);
             		$logger->debug('Read TimeOut :'.$options[RequestOptions::TIMEOUT]);
             	}
             	$logger->info('Sending Request to Service');
                $response = $this->client->send($request, $options);
            } catch (RequestException $e) {
            	$logger->info('Receive Response from Service : HTTP Status Code :'.$e->getCode());
            	if($e instanceof ConnectException){
					$logger->warn("[{$e->getCode()}] {$e->getMessage()}");
                    $isTimedOut = true;
                }
                throw new ApiException(
                    "[{$e->getCode()}] {$e->getMessage()}",
                    $e->getCode(),
                    $e->getResponse() ? $e->getResponse()->getHeaders() : null,
                    $e->getResponse() ? $e->getResponse()->getBody()->getContents() : null, CommonUtil::get_masked_string(CommonUtil::wrapHttpRequest($request, $_tempBody, $formParams)), $masked_authentication_data
                ,$isTimedOut);
            }

            $statusCode = $response->getStatusCode();
			$logger->info('Receive Response from Service : HTTP Status Code :'.$statusCode);
            if ($statusCode < 200 || $statusCode > 299) {
                throw new ApiException(
                    sprintf(
                        '[%d] Error connecting to the API (%s)',
                        $statusCode,
                        $request->getUri()
                    ),
                    $statusCode,
                    $response->getHeaders(),
                    $response->getBody(), CommonUtil::get_masked_string(CommonUtil::wrapHttpRequest($request, $_tempBody, $formParams)), $masked_authentication_data
                );
            }

            $responseBody = $response->getBody();
            if ($returnType === '\SplFileObject') {
                $content = $responseBody; //stream goes to serializer
            } else {
                $content = $responseBody->getContents();
                if ($returnType !== 'string') {
                    $content = json_decode($content);
                }
            }

            return [
                ObjectSerializer::deserialize($content, $returnType, []),
                $response->getStatusCode(),
                $response->getHeaders()
            ];

        } catch (ApiException $e) {
            switch ($e->getCode()) {
                case 201:
                    $data = ObjectSerializer::deserialize(
                        json_decode($e->getResponseBody()),
                        '\Orbipay\PaymentsApi\Client\Model\RecordedPaymentResponse',
                        $e->getResponseHeaders()
                    );
                    $e->setResponseObject($data);
                    break;
                case 400:
                    $data = ObjectSerializer::deserialize(
                        json_decode($e->getResponseBody()),
                        '\Orbipay\PaymentsApi\Client\Model\ErrorResponseVo',
                        $e->getResponseHeaders()
                    );
                    $e->setResponseObject($data);
                    break;
                case 409:
                    $data = ObjectSerializer::deserialize(
                        json_decode($e->getResponseBody()),
                        '\Orbipay\PaymentsApi\Client\Model\ErrorResponseVo',
                        $e->getResponseHeaders()
                    );
                    $e->setResponseObject($data);
                    break;
                case 422:
                    $data = ObjectSerializer::deserialize(
                        json_decode($e->getResponseBody()),
                        '\Orbipay\PaymentsApi\Client\Model\ErrorResponseVo',
                        $e->getResponseHeaders()
                    );
                    $e->setResponseObject($data);
                    break;
            }
            throw $e;
        }
    }


    /**
     * Create request for operation 'recordPayment'
     *
     * @param  string $channel The channel through which the API is invoked. (required)
     * @param  string $client_key The unique identifier assigned by EBPP to the client. (required)
     * @param  string $product The product identifier corresponding to the API. (required)
     * @param  string $timestamp The timestamp for the moment when the API request is created. (required)
     * @param  string $idempotent_request_key The unique token that clients can generate and maintain in order to identify an API request. (required)
     * @param  string $requestor_type Type of the requestor of the API. (required)
     * @param  RecordPaymentRequest $record_payment_request JSON containing all the fields required to record a payment. (required)
     * @param  string $requestor The identifier for the requestor of the API. (optional)
     * @param  string $x_opay_headers Intended for the future use. (optional)
     * @param  string $trace_id The unique reference that can be used for tracing and debugging an API call. (optional)
     *
     * @throws \InvalidArgumentException
     * @return array
     */
    protected function recordPaymentRequest($channel, $client_key, $product, $timestamp, $idempotent_request_key, $requestor_type, $record_payment_request, $requestor = null, $x_opay_headers = null, $trace_id = null)
    {
    
     $logger = LoggerFactory::getLogger(__CLASS__);
    
       
		$method = 'POST';
        $resourcePath = '/recordedpayments';
        $formParams = [];
        $queryParams = [];
        $headerParams = [];
        $httpBody = '';
        $multipart = false;
        $masked_authentication_data = '';

        // header params
        if ($channel !== null) {
            $headerParams['channel'] = ObjectSerializer::toHeaderValue($channel);
        }
        // header params
        if ($client_key !== null) {
            $headerParams['client_key'] = ObjectSerializer::toHeaderValue($client_key);
        }
        // header params
        if ($product !== null) {
            $headerParams['product'] = ObjectSerializer::toHeaderValue($product);
        }
        // header params
        if ($timestamp !== null) {
            $headerParams['timestamp'] = ObjectSerializer::toHeaderValue($timestamp);
        }
        // header params
        if ($idempotent_request_key !== null) {
            $headerParams['idempotent_request_key'] = ObjectSerializer::toHeaderValue($idempotent_request_key);
        }
        // header params
        if ($requestor_type !== null) {
            $headerParams['requestor_type'] = ObjectSerializer::toHeaderValue($requestor_type);
        }
        // header params
        if ($requestor !== null) {
            $headerParams['requestor'] = ObjectSerializer::toHeaderValue($requestor);
        }
        // header params
        if ($x_opay_headers !== null) {
            $headerParams['X-OPAY-Headers'] = ObjectSerializer::toHeaderValue($x_opay_headers);
        }
        // header params
        if ($trace_id !== null) {
            $headerParams['trace_id'] = ObjectSerializer::toHeaderValue($trace_id);
        }


        // body params
        $_tempBody = null;
        if (isset($record_payment_request)) {
            $_tempBody = $record_payment_request;
        }

        if ($multipart) {
            $headers = $this->headerSelector->selectHeadersForMultipart(
                ['application/json']
            );
        } else {
            $headers = $this->headerSelector->selectHeaders(
                ['application/json'],
                ['application/json']
            );
        }

        // for model (json/xml)
        if (isset($_tempBody)) {
            // $_tempBody is the method argument, if present
            $httpBody = $_tempBody;
            // \stdClass has no __toString(), so we should encode it manually
            if ($httpBody instanceof \stdClass && $headers['Content-Type'] === 'application/json') {
                $httpBody = \GuzzleHttp\json_encode($httpBody);
            }
        } elseif (count($formParams) > 0) {
            if ($multipart) {
                $multipartContents = [];
                foreach ($formParams as $formParamName => $formParamValue) {
                    $multipartContents[] = [
                        'name' => $formParamName,
                        'contents' => $formParamValue
                    ];
                }
                // for HTTP post (form)
                $httpBody = new MultipartStream($multipartContents);

            } elseif ($headers['Content-Type'] === 'application/json') {
                $httpBody = \GuzzleHttp\json_encode($formParams);

            } else {
                // for HTTP post (form)
                $httpBody = \GuzzleHttp\Psr7\build_query($formParams);
            }
        }
        
        if($logger->isDebugEnabled()){
            $logger->debug('HTTP Method     :'.$method);
            $logger->debug('HTTP URL        :'.$this->config->getHost() . $resourcePath);
        }
        
         if(empty($this->getConfig()->getClientApiKey())){
        	$this->getConfig()->setClientApiKey($this->getConfig()->getClientKey());
        }
        
        if(empty($this->getConfig()->getAuthScheme()) || $this->getConfig()->authScheme1 == $this->getConfig()->getAuthScheme()){
	         
	        list($hash, $masked_authentication_data) = AuthenticationUtil::computeOPAY1HMACSHA256Hash($method, $resourcePath, $headerParams, $queryParams, $formParams, $_tempBody, $this->getConfig()->getClientSignatureKey());
	        $headerParams['Authorization'] = $this->getConfig()->authScheme1 . " Credential=" . $this->getConfig()->getClientApiKey() . ",Signature=" . $hash;
        
        }elseif($this->getConfig()->authScheme2 == $this->getConfig()->getAuthScheme()){
				
	        $hash = AuthenticationUtil::computeOPAY2HMACSHA256Hash($this->getConfig()->getClientKey(), $this->getConfig()->getClientApiKey() ,$this->getConfig()->getClientSignatureKey(), $headerParams['timestamp']);
	        $headerParams['Authorization'] = $this->getConfig()->authScheme2 . " Credential=" . $this->getConfig()->getClientApiKey() . ",Signature=" . $hash;
	                
        }elseif($this->getConfig()->authScheme3 == $this->getConfig()->getAuthScheme()){
        	$headerParams['Authorization'] = $this->getConfig()->authScheme3 . " Credential=" . $this->getConfig()->getClientApiKey();
        }else{
        	$logger->error('Invalid auth_scheme received.');
           throw new \InvalidArgumentException('Invalid auth_scheme received.');
        }
        


        $defaultHeaders = [];
        if ($this->config->getUserAgent()) {
            $defaultHeaders['User-Agent'] = $this->config->getUserAgent();
        }

        $headers = array_merge(
            $defaultHeaders,
            $headerParams,
            $headers
        );

		if($logger->isDebugEnabled()){
			$logger->debug('Request Headers  :'.json_encode($headers));
		}
		
        $query = \GuzzleHttp\Psr7\build_query($queryParams);
        return [ new Request(
            $method,
            $this->config->getHost() . $resourcePath . ($query ? "?{$query}" : ''),
            $headers,
            $httpBody
        ), $_tempBody, $formParams, $masked_authentication_data];
    }

    /**
     * Operation retrieveFeePayment
     *
     * Get Fee Payment
     *
     * @param  string $channel The channel through which the API is invoked. (required)
     * @param  string $client_key The unique identifier assigned by OrbiPay to the client. (required)
     * @param  string $product The product identifier corresponding to the API. (required)
     * @param  string $timestamp The timestamp for the moment when the API request is created. (required)
     * @param  string $idempotent_request_key The unique token that clients can generate and maintain in order to identify an API request. (required)
     * @param  string $requestor_type Type of the requestor of the API. (required)
     * @param  string $id_fee The unique identifier assigned by OrbiPay to the FeePayment. (required)
     * @param  string $id_customer The unique identifier assigned by OrbiPay to the customer. (required)
     * @param  string $requestor The identifier for the requestor of the API. (optional)
     * @param  string $x_opay_headers Intended for the future use. (optional)
     * @param  string $trace_id The unique reference that can be used for tracing and debugging an API call. (optional)
     *
     * @throws \Orbipay\PaymentsApi\Client\ApiException on non-2xx response
     * @throws \InvalidArgumentException
     * @return array of \Orbipay\PaymentsApi\Client\Model\PaymentResponse, HTTP status code, HTTP response headers (array of strings)
     */
    public function retrieveFeePayment($channel, $client_key, $product, $timestamp, $idempotent_request_key, $requestor_type, $id_fee, $id_customer, $requestor = null, $x_opay_headers = null, $trace_id = null)
    {
        $response = $this->retrieveFeePaymentWithHttpInfo($channel, $client_key, $product, $timestamp, $idempotent_request_key, $requestor_type, $id_fee, $id_customer, $requestor, $x_opay_headers, $trace_id);
        return $response;
    }

    /**
     * Operation retrieveFeePaymentWithHttpInfo
     *
     * Get Fee Payment
     *
     * @param  string $channel The channel through which the API is invoked. (required)
     * @param  string $client_key The unique identifier assigned by OrbiPay to the client. (required)
     * @param  string $product The product identifier corresponding to the API. (required)
     * @param  string $timestamp The timestamp for the moment when the API request is created. (required)
     * @param  string $idempotent_request_key The unique token that clients can generate and maintain in order to identify an API request. (required)
     * @param  string $requestor_type Type of the requestor of the API. (required)
     * @param  string $id_fee The unique identifier assigned by OrbiPay to the FeePayment. (required)
     * @param  string $id_customer The unique identifier assigned by OrbiPay to the customer. (required)
     * @param  string $requestor The identifier for the requestor of the API. (optional)
     * @param  string $x_opay_headers Intended for the future use. (optional)
     * @param  string $trace_id The unique reference that can be used for tracing and debugging an API call. (optional)
     *
     * @throws \Orbipay\PaymentsApi\Client\ApiException on non-2xx response
     * @throws \InvalidArgumentException
     * @return array of \Orbipay\PaymentsApi\Client\Model\PaymentResponse, HTTP status code, HTTP response headers (array of strings)
     */
    public function retrieveFeePaymentWithHttpInfo($channel, $client_key, $product, $timestamp, $idempotent_request_key, $requestor_type, $id_fee, $id_customer, $requestor = null, $x_opay_headers = null, $trace_id = null)
    {
        $returnType = '\Orbipay\PaymentsApi\Client\Model\PaymentResponse';
        list($request, $_tempBody, $formParams, $masked_authentication_data) = $this->retrieveFeePaymentRequest($channel, $client_key, $product, $timestamp, $idempotent_request_key, $requestor_type, $id_fee, $id_customer, $requestor, $x_opay_headers, $trace_id);
		$logger = LoggerFactory::getLogger(__CLASS__);
        try {
            $options = $this->createHttpClientOption();
            try {
             	$isTimedOut = false;
             	if($logger->isDebugEnabled()){
             		$logger->debug('Connect TimeOut :'.$options[RequestOptions::TIMEOUT]);
             		$logger->debug('Read TimeOut :'.$options[RequestOptions::TIMEOUT]);
             	}
             	$logger->info('Sending Request to Service');
                $response = $this->client->send($request, $options);
            } catch (RequestException $e) {
            	$logger->info('Receive Response from Service : HTTP Status Code :'.$e->getCode());
            	if($e instanceof ConnectException){
					$logger->warn("[{$e->getCode()}] {$e->getMessage()}");
                    $isTimedOut = true;
                }
                throw new ApiException(
                    "[{$e->getCode()}] {$e->getMessage()}",
                    $e->getCode(),
                    $e->getResponse() ? $e->getResponse()->getHeaders() : null,
                    $e->getResponse() ? $e->getResponse()->getBody()->getContents() : null, CommonUtil::get_masked_string(CommonUtil::wrapHttpRequest($request, $_tempBody, $formParams)), $masked_authentication_data
                ,$isTimedOut);
            }

            $statusCode = $response->getStatusCode();
			$logger->info('Receive Response from Service : HTTP Status Code :'.$statusCode);
            if ($statusCode < 200 || $statusCode > 299) {
                throw new ApiException(
                    sprintf(
                        '[%d] Error connecting to the API (%s)',
                        $statusCode,
                        $request->getUri()
                    ),
                    $statusCode,
                    $response->getHeaders(),
                    $response->getBody(), CommonUtil::get_masked_string(CommonUtil::wrapHttpRequest($request, $_tempBody, $formParams)), $masked_authentication_data
                );
            }

            $responseBody = $response->getBody();
            if ($returnType === '\SplFileObject') {
                $content = $responseBody; //stream goes to serializer
            } else {
                $content = $responseBody->getContents();
                if ($returnType !== 'string') {
                    $content = json_decode($content);
                }
            }

            return [
                ObjectSerializer::deserialize($content, $returnType, []),
                $response->getStatusCode(),
                $response->getHeaders()
            ];

        } catch (ApiException $e) {
            switch ($e->getCode()) {
                case 200:
                    $data = ObjectSerializer::deserialize(
                        json_decode($e->getResponseBody()),
                        '\Orbipay\PaymentsApi\Client\Model\PaymentResponse',
                        $e->getResponseHeaders()
                    );
                    $e->setResponseObject($data);
                    break;
                case 400:
                    $data = ObjectSerializer::deserialize(
                        json_decode($e->getResponseBody()),
                        '\Orbipay\PaymentsApi\Client\Model\ErrorResponseVo',
                        $e->getResponseHeaders()
                    );
                    $e->setResponseObject($data);
                    break;
                case 409:
                    $data = ObjectSerializer::deserialize(
                        json_decode($e->getResponseBody()),
                        '\Orbipay\PaymentsApi\Client\Model\ErrorResponseVo',
                        $e->getResponseHeaders()
                    );
                    $e->setResponseObject($data);
                    break;
                case 422:
                    $data = ObjectSerializer::deserialize(
                        json_decode($e->getResponseBody()),
                        '\Orbipay\PaymentsApi\Client\Model\ErrorResponseVo',
                        $e->getResponseHeaders()
                    );
                    $e->setResponseObject($data);
                    break;
            }
            throw $e;
        }
    }


    /**
     * Create request for operation 'retrieveFeePayment'
     *
     * @param  string $channel The channel through which the API is invoked. (required)
     * @param  string $client_key The unique identifier assigned by OrbiPay to the client. (required)
     * @param  string $product The product identifier corresponding to the API. (required)
     * @param  string $timestamp The timestamp for the moment when the API request is created. (required)
     * @param  string $idempotent_request_key The unique token that clients can generate and maintain in order to identify an API request. (required)
     * @param  string $requestor_type Type of the requestor of the API. (required)
     * @param  string $id_fee The unique identifier assigned by OrbiPay to the FeePayment. (required)
     * @param  string $id_customer The unique identifier assigned by OrbiPay to the customer. (required)
     * @param  string $requestor The identifier for the requestor of the API. (optional)
     * @param  string $x_opay_headers Intended for the future use. (optional)
     * @param  string $trace_id The unique reference that can be used for tracing and debugging an API call. (optional)
     *
     * @throws \InvalidArgumentException
     * @return array
     */
    protected function retrieveFeePaymentRequest($channel, $client_key, $product, $timestamp, $idempotent_request_key, $requestor_type, $id_fee, $id_customer, $requestor = null, $x_opay_headers = null, $trace_id = null)
    {
    
     $logger = LoggerFactory::getLogger(__CLASS__);
    
        // verify the required parameter 'id_fee' is set
        if ($id_fee === null || (is_array($id_fee) && count($id_fee) === 0) || strlen($id_fee) === 0) {
     		$logger->error('Missing the required parameter $id_fee');
           throw new \InvalidArgumentException(
                 'Missing the required parameter $id_fee'
             );
        }
        // verify the required parameter 'id_customer' is set
        if ($id_customer === null || (is_array($id_customer) && count($id_customer) === 0) || strlen($id_customer) === 0) {
     		$logger->error('Missing the required parameter $id_customer');
           throw new \InvalidArgumentException(
                 'Missing the required parameter $id_customer'
             );
        }
       
		$method = 'GET';
        $resourcePath = '/customers/{ID_CUSTOMER}/fees/{ID_FEE}';
        $formParams = [];
        $queryParams = [];
        $headerParams = [];
        $httpBody = '';
        $multipart = false;
        $masked_authentication_data = '';

        // header params
        if ($channel !== null) {
            $headerParams['channel'] = ObjectSerializer::toHeaderValue($channel);
        }
        // header params
        if ($client_key !== null) {
            $headerParams['client_key'] = ObjectSerializer::toHeaderValue($client_key);
        }
        // header params
        if ($product !== null) {
            $headerParams['product'] = ObjectSerializer::toHeaderValue($product);
        }
        // header params
        if ($timestamp !== null) {
            $headerParams['timestamp'] = ObjectSerializer::toHeaderValue($timestamp);
        }
        // header params
        if ($idempotent_request_key !== null) {
            $headerParams['idempotent_request_key'] = ObjectSerializer::toHeaderValue($idempotent_request_key);
        }
        // header params
        if ($requestor_type !== null) {
            $headerParams['requestor_type'] = ObjectSerializer::toHeaderValue($requestor_type);
        }
        // header params
        if ($requestor !== null) {
            $headerParams['requestor'] = ObjectSerializer::toHeaderValue($requestor);
        }
        // header params
        if ($x_opay_headers !== null) {
            $headerParams['X-OPAY-Headers'] = ObjectSerializer::toHeaderValue($x_opay_headers);
        }
        // header params
        if ($trace_id !== null) {
            $headerParams['trace_id'] = ObjectSerializer::toHeaderValue($trace_id);
        }

        // path params
        if ($id_fee !== null) {
            $resourcePath = str_replace(
                '{' . 'ID_FEE' . '}',
                ObjectSerializer::toPathValue($id_fee),
                $resourcePath
            );
        }
        // path params
        if ($id_customer !== null) {
            $resourcePath = str_replace(
                '{' . 'ID_CUSTOMER' . '}',
                ObjectSerializer::toPathValue($id_customer),
                $resourcePath
            );
        }

        // body params
        $_tempBody = null;

        if ($multipart) {
            $headers = $this->headerSelector->selectHeadersForMultipart(
                ['application/json']
            );
        } else {
            $headers = $this->headerSelector->selectHeaders(
                ['application/json'],
                []
            );
        }

        // for model (json/xml)
        if (isset($_tempBody)) {
            // $_tempBody is the method argument, if present
            $httpBody = $_tempBody;
            // \stdClass has no __toString(), so we should encode it manually
            if ($httpBody instanceof \stdClass && $headers['Content-Type'] === 'application/json') {
                $httpBody = \GuzzleHttp\json_encode($httpBody);
            }
        } elseif (count($formParams) > 0) {
            if ($multipart) {
                $multipartContents = [];
                foreach ($formParams as $formParamName => $formParamValue) {
                    $multipartContents[] = [
                        'name' => $formParamName,
                        'contents' => $formParamValue
                    ];
                }
                // for HTTP post (form)
                $httpBody = new MultipartStream($multipartContents);

            } elseif ($headers['Content-Type'] === 'application/json') {
                $httpBody = \GuzzleHttp\json_encode($formParams);

            } else {
                // for HTTP post (form)
                $httpBody = \GuzzleHttp\Psr7\build_query($formParams);
            }
        }
        
        if($logger->isDebugEnabled()){
            $logger->debug('HTTP Method     :'.$method);
            $logger->debug('HTTP URL        :'.$this->config->getHost() . $resourcePath);
        }
        
         if(empty($this->getConfig()->getClientApiKey())){
        	$this->getConfig()->setClientApiKey($this->getConfig()->getClientKey());
        }
        
        if(empty($this->getConfig()->getAuthScheme()) || $this->getConfig()->authScheme1 == $this->getConfig()->getAuthScheme()){
	         
	        list($hash, $masked_authentication_data) = AuthenticationUtil::computeOPAY1HMACSHA256Hash($method, $resourcePath, $headerParams, $queryParams, $formParams, $_tempBody, $this->getConfig()->getClientSignatureKey());
	        $headerParams['Authorization'] = $this->getConfig()->authScheme1 . " Credential=" . $this->getConfig()->getClientApiKey() . ",Signature=" . $hash;
        
        }elseif($this->getConfig()->authScheme2 == $this->getConfig()->getAuthScheme()){
				
	        $hash = AuthenticationUtil::computeOPAY2HMACSHA256Hash($this->getConfig()->getClientKey(), $this->getConfig()->getClientApiKey() ,$this->getConfig()->getClientSignatureKey(), $headerParams['timestamp']);
	        $headerParams['Authorization'] = $this->getConfig()->authScheme2 . " Credential=" . $this->getConfig()->getClientApiKey() . ",Signature=" . $hash;
	                
        }elseif($this->getConfig()->authScheme3 == $this->getConfig()->getAuthScheme()){
        	$headerParams['Authorization'] = $this->getConfig()->authScheme3 . " Credential=" . $this->getConfig()->getClientApiKey();
        }else{
        	$logger->error('Invalid auth_scheme received.');
           throw new \InvalidArgumentException('Invalid auth_scheme received.');
        }
        


        $defaultHeaders = [];
        if ($this->config->getUserAgent()) {
            $defaultHeaders['User-Agent'] = $this->config->getUserAgent();
        }

        $headers = array_merge(
            $defaultHeaders,
            $headerParams,
            $headers
        );

		if($logger->isDebugEnabled()){
			$logger->debug('Request Headers  :'.json_encode($headers));
		}
		
        $query = \GuzzleHttp\Psr7\build_query($queryParams);
        return [ new Request(
            $method,
            $this->config->getHost() . $resourcePath . ($query ? "?{$query}" : ''),
            $headers,
            $httpBody
        ), $_tempBody, $formParams, $masked_authentication_data];
    }

    /**
     * Operation retrievePayments
     *
     * Retrieve Payments
     *
     * @param  string $channel The channel through which the API is invoked. (required)
     * @param  string $client_key The unique identifier assigned by EBPP to the client. (required)
     * @param  string $product The product identifier corresponding to the API. (required)
     * @param  string $timestamp The timestamp for the moment when the API request is created. (required)
     * @param  string $idempotent_request_key The unique token that clients can generate and maintain in order to identify an API request. (required)
     * @param  string $requestor_type Type of the requestor of the API. (required)
     * @param  string $requestor The identifier for the requestor of the API. (optional)
     * @param  string $x_opay_headers Intended for the future use. (optional)
     * @param  string $trace_id The unique reference that can be used for tracing and debugging an API call. (optional)
     * @param  string $id_customer The unique identifier assigned by EBPP to the customer. (optional)
     * @param  string $id_funding_account The unique identifier assigned by EBPP to the funding account (optional)
     * @param  string $id_customer_account The unique identifier assigned by EBPP to the customer account (optional)
     * @param  string $confirmation_number The confirmation number or reference provided to the customer for the successful payment. (optional)
     * @param  string[] $status The status of the payment. This can take multiple values in the format key&#x3D;value1&amp;key&#x3D;value2.... (optional)
     * @param  string[] $payment_schedule_type The schedule type for the payment setup. This can take multiple values in the format key&#x3D;value1&amp;key&#x3D;value2.... (optional)
     * @param  string[] $payment_method The fulfillment method for the payment. (optional)
     * @param  string $from_date The date from which payments, matching the criteria specified, need to be retrieved. (optional)
     * @param  string $to_date The date upto which payments, matching the criteria specified, need to be retrieved. (optional)
     * @param  string $page_size The maximum number of objects returned in the query. (optional)
     * @param  string $query_id query id of the Retrieve/Search Customers lookup. (optional)
     * @param  string $from_index To fetch the next set of objects that start after this object (optional)
     * @param  string $to_index To fetch the previous set of objects that end at this object (optional)
     *
     * @throws \Orbipay\PaymentsApi\Client\ApiException on non-2xx response
     * @throws \InvalidArgumentException
     * @return array of \Orbipay\PaymentsApi\Client\Model\PaymentsResponse, HTTP status code, HTTP response headers (array of strings)
     */
    public function retrievePayments($channel, $client_key, $product, $timestamp, $idempotent_request_key, $requestor_type, $requestor = null, $x_opay_headers = null, $trace_id = null, $id_customer = null, $id_funding_account = null, $id_customer_account = null, $confirmation_number = null, $status = null, $payment_schedule_type = null, $payment_method = null, $from_date = null, $to_date = null, $page_size = null, $query_id = null, $from_index = null, $to_index = null)
    {
        $response = $this->retrievePaymentsWithHttpInfo($channel, $client_key, $product, $timestamp, $idempotent_request_key, $requestor_type, $requestor, $x_opay_headers, $trace_id, $id_customer, $id_funding_account, $id_customer_account, $confirmation_number, $status, $payment_schedule_type, $payment_method, $from_date, $to_date, $page_size, $query_id, $from_index, $to_index);
        return $response;
    }

    /**
     * Operation retrievePaymentsWithHttpInfo
     *
     * Retrieve Payments
     *
     * @param  string $channel The channel through which the API is invoked. (required)
     * @param  string $client_key The unique identifier assigned by EBPP to the client. (required)
     * @param  string $product The product identifier corresponding to the API. (required)
     * @param  string $timestamp The timestamp for the moment when the API request is created. (required)
     * @param  string $idempotent_request_key The unique token that clients can generate and maintain in order to identify an API request. (required)
     * @param  string $requestor_type Type of the requestor of the API. (required)
     * @param  string $requestor The identifier for the requestor of the API. (optional)
     * @param  string $x_opay_headers Intended for the future use. (optional)
     * @param  string $trace_id The unique reference that can be used for tracing and debugging an API call. (optional)
     * @param  string $id_customer The unique identifier assigned by EBPP to the customer. (optional)
     * @param  string $id_funding_account The unique identifier assigned by EBPP to the funding account (optional)
     * @param  string $id_customer_account The unique identifier assigned by EBPP to the customer account (optional)
     * @param  string $confirmation_number The confirmation number or reference provided to the customer for the successful payment. (optional)
     * @param  string[] $status The status of the payment. This can take multiple values in the format key&#x3D;value1&amp;key&#x3D;value2.... (optional)
     * @param  string[] $payment_schedule_type The schedule type for the payment setup. This can take multiple values in the format key&#x3D;value1&amp;key&#x3D;value2.... (optional)
     * @param  string[] $payment_method The fulfillment method for the payment. (optional)
     * @param  string $from_date The date from which payments, matching the criteria specified, need to be retrieved. (optional)
     * @param  string $to_date The date upto which payments, matching the criteria specified, need to be retrieved. (optional)
     * @param  string $page_size The maximum number of objects returned in the query. (optional)
     * @param  string $query_id query id of the Retrieve/Search Customers lookup. (optional)
     * @param  string $from_index To fetch the next set of objects that start after this object (optional)
     * @param  string $to_index To fetch the previous set of objects that end at this object (optional)
     *
     * @throws \Orbipay\PaymentsApi\Client\ApiException on non-2xx response
     * @throws \InvalidArgumentException
     * @return array of \Orbipay\PaymentsApi\Client\Model\PaymentsResponse, HTTP status code, HTTP response headers (array of strings)
     */
    public function retrievePaymentsWithHttpInfo($channel, $client_key, $product, $timestamp, $idempotent_request_key, $requestor_type, $requestor = null, $x_opay_headers = null, $trace_id = null, $id_customer = null, $id_funding_account = null, $id_customer_account = null, $confirmation_number = null, $status = null, $payment_schedule_type = null, $payment_method = null, $from_date = null, $to_date = null, $page_size = null, $query_id = null, $from_index = null, $to_index = null)
    {
        $returnType = '\Orbipay\PaymentsApi\Client\Model\PaymentsResponse';
        list($request, $_tempBody, $formParams, $masked_authentication_data) = $this->retrievePaymentsRequest($channel, $client_key, $product, $timestamp, $idempotent_request_key, $requestor_type, $requestor, $x_opay_headers, $trace_id, $id_customer, $id_funding_account, $id_customer_account, $confirmation_number, $status, $payment_schedule_type, $payment_method, $from_date, $to_date, $page_size, $query_id, $from_index, $to_index);
		$logger = LoggerFactory::getLogger(__CLASS__);
        try {
            $options = $this->createHttpClientOption();
            try {
             	$isTimedOut = false;
             	if($logger->isDebugEnabled()){
             		$logger->debug('Connect TimeOut :'.$options[RequestOptions::TIMEOUT]);
             		$logger->debug('Read TimeOut :'.$options[RequestOptions::TIMEOUT]);
             	}
             	$logger->info('Sending Request to Service');
                $response = $this->client->send($request, $options);
            } catch (RequestException $e) {
            	$logger->info('Receive Response from Service : HTTP Status Code :'.$e->getCode());
            	if($e instanceof ConnectException){
					$logger->warn("[{$e->getCode()}] {$e->getMessage()}");
                    $isTimedOut = true;
                }
                throw new ApiException(
                    "[{$e->getCode()}] {$e->getMessage()}",
                    $e->getCode(),
                    $e->getResponse() ? $e->getResponse()->getHeaders() : null,
                    $e->getResponse() ? $e->getResponse()->getBody()->getContents() : null, CommonUtil::get_masked_string(CommonUtil::wrapHttpRequest($request, $_tempBody, $formParams)), $masked_authentication_data
                ,$isTimedOut);
            }

            $statusCode = $response->getStatusCode();
			$logger->info('Receive Response from Service : HTTP Status Code :'.$statusCode);
            if ($statusCode < 200 || $statusCode > 299) {
                throw new ApiException(
                    sprintf(
                        '[%d] Error connecting to the API (%s)',
                        $statusCode,
                        $request->getUri()
                    ),
                    $statusCode,
                    $response->getHeaders(),
                    $response->getBody(), CommonUtil::get_masked_string(CommonUtil::wrapHttpRequest($request, $_tempBody, $formParams)), $masked_authentication_data
                );
            }

            $responseBody = $response->getBody();
            if ($returnType === '\SplFileObject') {
                $content = $responseBody; //stream goes to serializer
            } else {
                $content = $responseBody->getContents();
                if ($returnType !== 'string') {
                    $content = json_decode($content);
                }
            }

            return [
                ObjectSerializer::deserialize($content, $returnType, []),
                $response->getStatusCode(),
                $response->getHeaders()
            ];

        } catch (ApiException $e) {
            switch ($e->getCode()) {
                case 200:
                    $data = ObjectSerializer::deserialize(
                        json_decode($e->getResponseBody()),
                        '\Orbipay\PaymentsApi\Client\Model\PaymentsResponse',
                        $e->getResponseHeaders()
                    );
                    $e->setResponseObject($data);
                    break;
                case 400:
                    $data = ObjectSerializer::deserialize(
                        json_decode($e->getResponseBody()),
                        '\Orbipay\PaymentsApi\Client\Model\ErrorResponseVo',
                        $e->getResponseHeaders()
                    );
                    $e->setResponseObject($data);
                    break;
                case 409:
                    $data = ObjectSerializer::deserialize(
                        json_decode($e->getResponseBody()),
                        '\Orbipay\PaymentsApi\Client\Model\ErrorResponseVo',
                        $e->getResponseHeaders()
                    );
                    $e->setResponseObject($data);
                    break;
                case 422:
                    $data = ObjectSerializer::deserialize(
                        json_decode($e->getResponseBody()),
                        '\Orbipay\PaymentsApi\Client\Model\ErrorResponseVo',
                        $e->getResponseHeaders()
                    );
                    $e->setResponseObject($data);
                    break;
            }
            throw $e;
        }
    }


    /**
     * Create request for operation 'retrievePayments'
     *
     * @param  string $channel The channel through which the API is invoked. (required)
     * @param  string $client_key The unique identifier assigned by EBPP to the client. (required)
     * @param  string $product The product identifier corresponding to the API. (required)
     * @param  string $timestamp The timestamp for the moment when the API request is created. (required)
     * @param  string $idempotent_request_key The unique token that clients can generate and maintain in order to identify an API request. (required)
     * @param  string $requestor_type Type of the requestor of the API. (required)
     * @param  string $requestor The identifier for the requestor of the API. (optional)
     * @param  string $x_opay_headers Intended for the future use. (optional)
     * @param  string $trace_id The unique reference that can be used for tracing and debugging an API call. (optional)
     * @param  string $id_customer The unique identifier assigned by EBPP to the customer. (optional)
     * @param  string $id_funding_account The unique identifier assigned by EBPP to the funding account (optional)
     * @param  string $id_customer_account The unique identifier assigned by EBPP to the customer account (optional)
     * @param  string $confirmation_number The confirmation number or reference provided to the customer for the successful payment. (optional)
     * @param  string[] $status The status of the payment. This can take multiple values in the format key&#x3D;value1&amp;key&#x3D;value2.... (optional)
     * @param  string[] $payment_schedule_type The schedule type for the payment setup. This can take multiple values in the format key&#x3D;value1&amp;key&#x3D;value2.... (optional)
     * @param  string[] $payment_method The fulfillment method for the payment. (optional)
     * @param  string $from_date The date from which payments, matching the criteria specified, need to be retrieved. (optional)
     * @param  string $to_date The date upto which payments, matching the criteria specified, need to be retrieved. (optional)
     * @param  string $page_size The maximum number of objects returned in the query. (optional)
     * @param  string $query_id query id of the Retrieve/Search Customers lookup. (optional)
     * @param  string $from_index To fetch the next set of objects that start after this object (optional)
     * @param  string $to_index To fetch the previous set of objects that end at this object (optional)
     *
     * @throws \InvalidArgumentException
     * @return array
     */
    protected function retrievePaymentsRequest($channel, $client_key, $product, $timestamp, $idempotent_request_key, $requestor_type, $requestor = null, $x_opay_headers = null, $trace_id = null, $id_customer = null, $id_funding_account = null, $id_customer_account = null, $confirmation_number = null, $status = null, $payment_schedule_type = null, $payment_method = null, $from_date = null, $to_date = null, $page_size = null, $query_id = null, $from_index = null, $to_index = null)
    {
    
     $logger = LoggerFactory::getLogger(__CLASS__);
    
       
		$method = 'GET';
        $resourcePath = '/payments';
        $formParams = [];
        $queryParams = [];
        $headerParams = [];
        $httpBody = '';
        $multipart = false;
        $masked_authentication_data = '';

        // query params
        if ($id_customer !== null) {
            $queryParams['id_customer'] = ObjectSerializer::toQueryValue($id_customer);
        }
        // query params
        if ($id_funding_account !== null) {
            $queryParams['id_funding_account'] = ObjectSerializer::toQueryValue($id_funding_account);
        }
        // query params
        if ($id_customer_account !== null) {
            $queryParams['id_customer_account'] = ObjectSerializer::toQueryValue($id_customer_account);
        }
        // query params
        if ($confirmation_number !== null) {
            $queryParams['confirmation_number'] = ObjectSerializer::toQueryValue($confirmation_number);
        }
        // query params
        if ($status !== null) {
            $queryParams['status'] = ObjectSerializer::toQueryValue($status);
        }
        // query params
        if ($payment_schedule_type !== null) {
            $queryParams['payment_schedule_type'] = ObjectSerializer::toQueryValue($payment_schedule_type);
        }
        // query params
        if ($payment_method !== null) {
            $queryParams['payment_method'] = ObjectSerializer::toQueryValue($payment_method);
        }
        // query params
        if ($from_date !== null) {
            $queryParams['from_date'] = ObjectSerializer::toQueryValue($from_date);
        }
        // query params
        if ($to_date !== null) {
            $queryParams['to_date'] = ObjectSerializer::toQueryValue($to_date);
        }
        // query params
        if ($page_size !== null) {
            $queryParams['page_size'] = ObjectSerializer::toQueryValue($page_size);
        }
        // query params
        if ($query_id !== null) {
            $queryParams['query_id'] = ObjectSerializer::toQueryValue($query_id);
        }
        // query params
        if ($from_index !== null) {
            $queryParams['from_index'] = ObjectSerializer::toQueryValue($from_index);
        }
        // query params
        if ($to_index !== null) {
            $queryParams['to_index'] = ObjectSerializer::toQueryValue($to_index);
        }
        // header params
        if ($channel !== null) {
            $headerParams['channel'] = ObjectSerializer::toHeaderValue($channel);
        }
        // header params
        if ($client_key !== null) {
            $headerParams['client_key'] = ObjectSerializer::toHeaderValue($client_key);
        }
        // header params
        if ($product !== null) {
            $headerParams['product'] = ObjectSerializer::toHeaderValue($product);
        }
        // header params
        if ($timestamp !== null) {
            $headerParams['timestamp'] = ObjectSerializer::toHeaderValue($timestamp);
        }
        // header params
        if ($idempotent_request_key !== null) {
            $headerParams['idempotent_request_key'] = ObjectSerializer::toHeaderValue($idempotent_request_key);
        }
        // header params
        if ($requestor_type !== null) {
            $headerParams['requestor_type'] = ObjectSerializer::toHeaderValue($requestor_type);
        }
        // header params
        if ($requestor !== null) {
            $headerParams['requestor'] = ObjectSerializer::toHeaderValue($requestor);
        }
        // header params
        if ($x_opay_headers !== null) {
            $headerParams['X-OPAY-Headers'] = ObjectSerializer::toHeaderValue($x_opay_headers);
        }
        // header params
        if ($trace_id !== null) {
            $headerParams['trace_id'] = ObjectSerializer::toHeaderValue($trace_id);
        }


        // body params
        $_tempBody = null;

        if ($multipart) {
            $headers = $this->headerSelector->selectHeadersForMultipart(
                ['application/json']
            );
        } else {
            $headers = $this->headerSelector->selectHeaders(
                ['application/json'],
                []
            );
        }

        // for model (json/xml)
        if (isset($_tempBody)) {
            // $_tempBody is the method argument, if present
            $httpBody = $_tempBody;
            // \stdClass has no __toString(), so we should encode it manually
            if ($httpBody instanceof \stdClass && $headers['Content-Type'] === 'application/json') {
                $httpBody = \GuzzleHttp\json_encode($httpBody);
            }
        } elseif (count($formParams) > 0) {
            if ($multipart) {
                $multipartContents = [];
                foreach ($formParams as $formParamName => $formParamValue) {
                    $multipartContents[] = [
                        'name' => $formParamName,
                        'contents' => $formParamValue
                    ];
                }
                // for HTTP post (form)
                $httpBody = new MultipartStream($multipartContents);

            } elseif ($headers['Content-Type'] === 'application/json') {
                $httpBody = \GuzzleHttp\json_encode($formParams);

            } else {
                // for HTTP post (form)
                $httpBody = \GuzzleHttp\Psr7\build_query($formParams);
            }
        }
        
        if($logger->isDebugEnabled()){
            $logger->debug('HTTP Method     :'.$method);
            $logger->debug('HTTP URL        :'.$this->config->getHost() . $resourcePath);
        }
        
         if(empty($this->getConfig()->getClientApiKey())){
        	$this->getConfig()->setClientApiKey($this->getConfig()->getClientKey());
        }
        
        if(empty($this->getConfig()->getAuthScheme()) || $this->getConfig()->authScheme1 == $this->getConfig()->getAuthScheme()){
	         
	        list($hash, $masked_authentication_data) = AuthenticationUtil::computeOPAY1HMACSHA256Hash($method, $resourcePath, $headerParams, $queryParams, $formParams, $_tempBody, $this->getConfig()->getClientSignatureKey());
	        $headerParams['Authorization'] = $this->getConfig()->authScheme1 . " Credential=" . $this->getConfig()->getClientApiKey() . ",Signature=" . $hash;
        
        }elseif($this->getConfig()->authScheme2 == $this->getConfig()->getAuthScheme()){
				
	        $hash = AuthenticationUtil::computeOPAY2HMACSHA256Hash($this->getConfig()->getClientKey(), $this->getConfig()->getClientApiKey() ,$this->getConfig()->getClientSignatureKey(), $headerParams['timestamp']);
	        $headerParams['Authorization'] = $this->getConfig()->authScheme2 . " Credential=" . $this->getConfig()->getClientApiKey() . ",Signature=" . $hash;
	                
        }elseif($this->getConfig()->authScheme3 == $this->getConfig()->getAuthScheme()){
        	$headerParams['Authorization'] = $this->getConfig()->authScheme3 . " Credential=" . $this->getConfig()->getClientApiKey();
        }else{
        	$logger->error('Invalid auth_scheme received.');
           throw new \InvalidArgumentException('Invalid auth_scheme received.');
        }
        


        $defaultHeaders = [];
        if ($this->config->getUserAgent()) {
            $defaultHeaders['User-Agent'] = $this->config->getUserAgent();
        }

        $headers = array_merge(
            $defaultHeaders,
            $headerParams,
            $headers
        );

		if($logger->isDebugEnabled()){
			$logger->debug('Request Headers  :'.json_encode($headers));
		}
		
        $query = \GuzzleHttp\Psr7\build_query($queryParams);
        return [ new Request(
            $method,
            $this->config->getHost() . $resourcePath . ($query ? "?{$query}" : ''),
            $headers,
            $httpBody
        ), $_tempBody, $formParams, $masked_authentication_data];
    }

    /**
     * Operation simplePayment
     *
     * Simple Payment
     *
     * @param  string $channel The channel through which the API is invoked. (required)
     * @param  string $client_key The unique identifier assigned by EBPP to the client. (required)
     * @param  string $product The product identifier corresponding to the API. (required)
     * @param  string $timestamp The timestamp for the moment when the API request is created. (required)
     * @param  string $idempotent_request_key The unique token that clients can generate and maintain in order to identify an API request. (required)
     * @param  string $requestor_type Type of the requestor of the API. (required)
     * @param  SimplePaymentRequest $simple_payment_request JSON containing all the fields required to make a payment. (required)
     * @param  string $requestor The identifier for the requestor of the API. (optional)
     * @param  string $x_opay_headers Intended for the future use. (optional)
     * @param  string $trace_id The unique reference that can be used for tracing and debugging an API call. (optional)
     * @param  string $save_funding_account query param (optional)
     *
     * @throws \Orbipay\PaymentsApi\Client\ApiException on non-2xx response
     * @throws \InvalidArgumentException
     * @return array of \Orbipay\PaymentsApi\Client\Model\SimplePaymentResponse, HTTP status code, HTTP response headers (array of strings)
     */
    public function simplePayment($channel, $client_key, $product, $timestamp, $idempotent_request_key, $requestor_type, $simple_payment_request, $requestor = null, $x_opay_headers = null, $trace_id = null, $save_funding_account = null)
    {
        $response = $this->simplePaymentWithHttpInfo($channel, $client_key, $product, $timestamp, $idempotent_request_key, $requestor_type, $simple_payment_request, $requestor, $x_opay_headers, $trace_id, $save_funding_account);
        return $response;
    }

    /**
     * Operation simplePaymentWithHttpInfo
     *
     * Simple Payment
     *
     * @param  string $channel The channel through which the API is invoked. (required)
     * @param  string $client_key The unique identifier assigned by EBPP to the client. (required)
     * @param  string $product The product identifier corresponding to the API. (required)
     * @param  string $timestamp The timestamp for the moment when the API request is created. (required)
     * @param  string $idempotent_request_key The unique token that clients can generate and maintain in order to identify an API request. (required)
     * @param  string $requestor_type Type of the requestor of the API. (required)
     * @param  SimplePaymentRequest $simple_payment_request JSON containing all the fields required to make a payment. (required)
     * @param  string $requestor The identifier for the requestor of the API. (optional)
     * @param  string $x_opay_headers Intended for the future use. (optional)
     * @param  string $trace_id The unique reference that can be used for tracing and debugging an API call. (optional)
     * @param  string $save_funding_account query param (optional)
     *
     * @throws \Orbipay\PaymentsApi\Client\ApiException on non-2xx response
     * @throws \InvalidArgumentException
     * @return array of \Orbipay\PaymentsApi\Client\Model\SimplePaymentResponse, HTTP status code, HTTP response headers (array of strings)
     */
    public function simplePaymentWithHttpInfo($channel, $client_key, $product, $timestamp, $idempotent_request_key, $requestor_type, $simple_payment_request, $requestor = null, $x_opay_headers = null, $trace_id = null, $save_funding_account = null)
    {
        $returnType = '\Orbipay\PaymentsApi\Client\Model\SimplePaymentResponse';
        list($request, $_tempBody, $formParams, $masked_authentication_data) = $this->simplePaymentRequest($channel, $client_key, $product, $timestamp, $idempotent_request_key, $requestor_type, $simple_payment_request, $requestor, $x_opay_headers, $trace_id, $save_funding_account);
		$logger = LoggerFactory::getLogger(__CLASS__);
        try {
            $options = $this->createHttpClientOption();
            try {
             	$isTimedOut = false;
             	if($logger->isDebugEnabled()){
             		$logger->debug('Connect TimeOut :'.$options[RequestOptions::TIMEOUT]);
             		$logger->debug('Read TimeOut :'.$options[RequestOptions::TIMEOUT]);
             	}
             	$logger->info('Sending Request to Service');
                $response = $this->client->send($request, $options);
            } catch (RequestException $e) {
            	$logger->info('Receive Response from Service : HTTP Status Code :'.$e->getCode());
            	if($e instanceof ConnectException){
					$logger->warn("[{$e->getCode()}] {$e->getMessage()}");
                    $isTimedOut = true;
                }
                throw new ApiException(
                    "[{$e->getCode()}] {$e->getMessage()}",
                    $e->getCode(),
                    $e->getResponse() ? $e->getResponse()->getHeaders() : null,
                    $e->getResponse() ? $e->getResponse()->getBody()->getContents() : null, CommonUtil::get_masked_string(CommonUtil::wrapHttpRequest($request, $_tempBody, $formParams)), $masked_authentication_data
                ,$isTimedOut);
            }

            $statusCode = $response->getStatusCode();
			$logger->info('Receive Response from Service : HTTP Status Code :'.$statusCode);
            if ($statusCode < 200 || $statusCode > 299) {
                throw new ApiException(
                    sprintf(
                        '[%d] Error connecting to the API (%s)',
                        $statusCode,
                        $request->getUri()
                    ),
                    $statusCode,
                    $response->getHeaders(),
                    $response->getBody(), CommonUtil::get_masked_string(CommonUtil::wrapHttpRequest($request, $_tempBody, $formParams)), $masked_authentication_data
                );
            }

            $responseBody = $response->getBody();
            if ($returnType === '\SplFileObject') {
                $content = $responseBody; //stream goes to serializer
            } else {
                $content = $responseBody->getContents();
                if ($returnType !== 'string') {
                    $content = json_decode($content);
                }
            }

            return [
                ObjectSerializer::deserialize($content, $returnType, []),
                $response->getStatusCode(),
                $response->getHeaders()
            ];

        } catch (ApiException $e) {
            switch ($e->getCode()) {
                case 201:
                    $data = ObjectSerializer::deserialize(
                        json_decode($e->getResponseBody()),
                        '\Orbipay\PaymentsApi\Client\Model\SimplePaymentResponse',
                        $e->getResponseHeaders()
                    );
                    $e->setResponseObject($data);
                    break;
                case 400:
                    $data = ObjectSerializer::deserialize(
                        json_decode($e->getResponseBody()),
                        '\Orbipay\PaymentsApi\Client\Model\ErrorResponseVo',
                        $e->getResponseHeaders()
                    );
                    $e->setResponseObject($data);
                    break;
                case 409:
                    $data = ObjectSerializer::deserialize(
                        json_decode($e->getResponseBody()),
                        '\Orbipay\PaymentsApi\Client\Model\ErrorResponseVo',
                        $e->getResponseHeaders()
                    );
                    $e->setResponseObject($data);
                    break;
                case 422:
                    $data = ObjectSerializer::deserialize(
                        json_decode($e->getResponseBody()),
                        '\Orbipay\PaymentsApi\Client\Model\ErrorResponseVo',
                        $e->getResponseHeaders()
                    );
                    $e->setResponseObject($data);
                    break;
            }
            throw $e;
        }
    }


    /**
     * Create request for operation 'simplePayment'
     *
     * @param  string $channel The channel through which the API is invoked. (required)
     * @param  string $client_key The unique identifier assigned by EBPP to the client. (required)
     * @param  string $product The product identifier corresponding to the API. (required)
     * @param  string $timestamp The timestamp for the moment when the API request is created. (required)
     * @param  string $idempotent_request_key The unique token that clients can generate and maintain in order to identify an API request. (required)
     * @param  string $requestor_type Type of the requestor of the API. (required)
     * @param  SimplePaymentRequest $simple_payment_request JSON containing all the fields required to make a payment. (required)
     * @param  string $requestor The identifier for the requestor of the API. (optional)
     * @param  string $x_opay_headers Intended for the future use. (optional)
     * @param  string $trace_id The unique reference that can be used for tracing and debugging an API call. (optional)
     * @param  string $save_funding_account query param (optional)
     *
     * @throws \InvalidArgumentException
     * @return array
     */
    protected function simplePaymentRequest($channel, $client_key, $product, $timestamp, $idempotent_request_key, $requestor_type, $simple_payment_request, $requestor = null, $x_opay_headers = null, $trace_id = null, $save_funding_account = null)
    {
    
     $logger = LoggerFactory::getLogger(__CLASS__);
    
       
		$method = 'POST';
        $resourcePath = '/payments';
        $formParams = [];
        $queryParams = [];
        $headerParams = [];
        $httpBody = '';
        $multipart = false;
        $masked_authentication_data = '';

        // query params
        if ($save_funding_account !== null) {
            $queryParams['save_funding_account'] = ObjectSerializer::toQueryValue($save_funding_account);
        }
        // header params
        if ($channel !== null) {
            $headerParams['channel'] = ObjectSerializer::toHeaderValue($channel);
        }
        // header params
        if ($client_key !== null) {
            $headerParams['client_key'] = ObjectSerializer::toHeaderValue($client_key);
        }
        // header params
        if ($product !== null) {
            $headerParams['product'] = ObjectSerializer::toHeaderValue($product);
        }
        // header params
        if ($timestamp !== null) {
            $headerParams['timestamp'] = ObjectSerializer::toHeaderValue($timestamp);
        }
        // header params
        if ($idempotent_request_key !== null) {
            $headerParams['idempotent_request_key'] = ObjectSerializer::toHeaderValue($idempotent_request_key);
        }
        // header params
        if ($requestor_type !== null) {
            $headerParams['requestor_type'] = ObjectSerializer::toHeaderValue($requestor_type);
        }
        // header params
        if ($requestor !== null) {
            $headerParams['requestor'] = ObjectSerializer::toHeaderValue($requestor);
        }
        // header params
        if ($x_opay_headers !== null) {
            $headerParams['X-OPAY-Headers'] = ObjectSerializer::toHeaderValue($x_opay_headers);
        }
        // header params
        if ($trace_id !== null) {
            $headerParams['trace_id'] = ObjectSerializer::toHeaderValue($trace_id);
        }


        // body params
        $_tempBody = null;
        if (isset($simple_payment_request)) {
            $_tempBody = $simple_payment_request;
        }

        if ($multipart) {
            $headers = $this->headerSelector->selectHeadersForMultipart(
                ['application/json']
            );
        } else {
            $headers = $this->headerSelector->selectHeaders(
                ['application/json'],
                ['application/json']
            );
        }

        // for model (json/xml)
        if (isset($_tempBody)) {
            // $_tempBody is the method argument, if present
            $httpBody = $_tempBody;
            // \stdClass has no __toString(), so we should encode it manually
            if ($httpBody instanceof \stdClass && $headers['Content-Type'] === 'application/json') {
                $httpBody = \GuzzleHttp\json_encode($httpBody);
            }
        } elseif (count($formParams) > 0) {
            if ($multipart) {
                $multipartContents = [];
                foreach ($formParams as $formParamName => $formParamValue) {
                    $multipartContents[] = [
                        'name' => $formParamName,
                        'contents' => $formParamValue
                    ];
                }
                // for HTTP post (form)
                $httpBody = new MultipartStream($multipartContents);

            } elseif ($headers['Content-Type'] === 'application/json') {
                $httpBody = \GuzzleHttp\json_encode($formParams);

            } else {
                // for HTTP post (form)
                $httpBody = \GuzzleHttp\Psr7\build_query($formParams);
            }
        }
        
        if($logger->isDebugEnabled()){
            $logger->debug('HTTP Method     :'.$method);
            $logger->debug('HTTP URL        :'.$this->config->getHost() . $resourcePath);
        }
        
         if(empty($this->getConfig()->getClientApiKey())){
        	$this->getConfig()->setClientApiKey($this->getConfig()->getClientKey());
        }
        
        if(empty($this->getConfig()->getAuthScheme()) || $this->getConfig()->authScheme1 == $this->getConfig()->getAuthScheme()){
	         
	        list($hash, $masked_authentication_data) = AuthenticationUtil::computeOPAY1HMACSHA256Hash($method, $resourcePath, $headerParams, $queryParams, $formParams, $_tempBody, $this->getConfig()->getClientSignatureKey());
	        $headerParams['Authorization'] = $this->getConfig()->authScheme1 . " Credential=" . $this->getConfig()->getClientApiKey() . ",Signature=" . $hash;
        
        }elseif($this->getConfig()->authScheme2 == $this->getConfig()->getAuthScheme()){
				
	        $hash = AuthenticationUtil::computeOPAY2HMACSHA256Hash($this->getConfig()->getClientKey(), $this->getConfig()->getClientApiKey() ,$this->getConfig()->getClientSignatureKey(), $headerParams['timestamp']);
	        $headerParams['Authorization'] = $this->getConfig()->authScheme2 . " Credential=" . $this->getConfig()->getClientApiKey() . ",Signature=" . $hash;
	                
        }elseif($this->getConfig()->authScheme3 == $this->getConfig()->getAuthScheme()){
        	$headerParams['Authorization'] = $this->getConfig()->authScheme3 . " Credential=" . $this->getConfig()->getClientApiKey();
        }else{
        	$logger->error('Invalid auth_scheme received.');
           throw new \InvalidArgumentException('Invalid auth_scheme received.');
        }
        


        $defaultHeaders = [];
        if ($this->config->getUserAgent()) {
            $defaultHeaders['User-Agent'] = $this->config->getUserAgent();
        }

        $headers = array_merge(
            $defaultHeaders,
            $headerParams,
            $headers
        );

		if($logger->isDebugEnabled()){
			$logger->debug('Request Headers  :'.json_encode($headers));
		}
		
        $query = \GuzzleHttp\Psr7\build_query($queryParams);
        return [ new Request(
            $method,
            $this->config->getHost() . $resourcePath . ($query ? "?{$query}" : ''),
            $headers,
            $httpBody
        ), $_tempBody, $formParams, $masked_authentication_data];
    }

    /**
     * Operation updatePayment
     *
     * Update Payment
     *
     * @param  string $channel The channel through which the API is invoked. (required)
     * @param  string $client_key The unique identifier assigned by EBPP to the client. (required)
     * @param  string $product The product identifier corresponding to the API. (required)
     * @param  string $timestamp The timestamp for the moment when the API request is created. (required)
     * @param  string $idempotent_request_key The unique token that clients can generate and maintain in order to identify an API request. (required)
     * @param  string $requestor_type Type of the requestor of the API. (required)
     * @param  string $id_payment The unique identifier assigned by EBPP to the payment. (required)
     * @param  string $id_customer The unique identifier assigned by EBPP to the customer. (required)
     * @param  UpdatePaymentRequest $update_payment_request Details of the Payment to be updated. (required)
     * @param  string $requestor The identifier for the requestor of the API. (optional)
     * @param  string $x_opay_headers Intended for the future use. (optional)
     * @param  string $trace_id The unique reference that can be used for tracing and debugging an API call. (optional)
     *
     * @throws \Orbipay\PaymentsApi\Client\ApiException on non-2xx response
     * @throws \InvalidArgumentException
     * @return array of \Orbipay\PaymentsApi\Client\Model\PaymentResponse, HTTP status code, HTTP response headers (array of strings)
     */
    public function updatePayment($channel, $client_key, $product, $timestamp, $idempotent_request_key, $requestor_type, $id_payment, $id_customer, $update_payment_request, $requestor = null, $x_opay_headers = null, $trace_id = null)
    {
        $response = $this->updatePaymentWithHttpInfo($channel, $client_key, $product, $timestamp, $idempotent_request_key, $requestor_type, $id_payment, $id_customer, $update_payment_request, $requestor, $x_opay_headers, $trace_id);
        return $response;
    }

    /**
     * Operation updatePaymentWithHttpInfo
     *
     * Update Payment
     *
     * @param  string $channel The channel through which the API is invoked. (required)
     * @param  string $client_key The unique identifier assigned by EBPP to the client. (required)
     * @param  string $product The product identifier corresponding to the API. (required)
     * @param  string $timestamp The timestamp for the moment when the API request is created. (required)
     * @param  string $idempotent_request_key The unique token that clients can generate and maintain in order to identify an API request. (required)
     * @param  string $requestor_type Type of the requestor of the API. (required)
     * @param  string $id_payment The unique identifier assigned by EBPP to the payment. (required)
     * @param  string $id_customer The unique identifier assigned by EBPP to the customer. (required)
     * @param  UpdatePaymentRequest $update_payment_request Details of the Payment to be updated. (required)
     * @param  string $requestor The identifier for the requestor of the API. (optional)
     * @param  string $x_opay_headers Intended for the future use. (optional)
     * @param  string $trace_id The unique reference that can be used for tracing and debugging an API call. (optional)
     *
     * @throws \Orbipay\PaymentsApi\Client\ApiException on non-2xx response
     * @throws \InvalidArgumentException
     * @return array of \Orbipay\PaymentsApi\Client\Model\PaymentResponse, HTTP status code, HTTP response headers (array of strings)
     */
    public function updatePaymentWithHttpInfo($channel, $client_key, $product, $timestamp, $idempotent_request_key, $requestor_type, $id_payment, $id_customer, $update_payment_request, $requestor = null, $x_opay_headers = null, $trace_id = null)
    {
        $returnType = '\Orbipay\PaymentsApi\Client\Model\PaymentResponse';
        list($request, $_tempBody, $formParams, $masked_authentication_data) = $this->updatePaymentRequest($channel, $client_key, $product, $timestamp, $idempotent_request_key, $requestor_type, $id_payment, $id_customer, $update_payment_request, $requestor, $x_opay_headers, $trace_id);
		$logger = LoggerFactory::getLogger(__CLASS__);
        try {
            $options = $this->createHttpClientOption();
            try {
             	$isTimedOut = false;
             	if($logger->isDebugEnabled()){
             		$logger->debug('Connect TimeOut :'.$options[RequestOptions::TIMEOUT]);
             		$logger->debug('Read TimeOut :'.$options[RequestOptions::TIMEOUT]);
             	}
             	$logger->info('Sending Request to Service');
                $response = $this->client->send($request, $options);
            } catch (RequestException $e) {
            	$logger->info('Receive Response from Service : HTTP Status Code :'.$e->getCode());
            	if($e instanceof ConnectException){
					$logger->warn("[{$e->getCode()}] {$e->getMessage()}");
                    $isTimedOut = true;
                }
                throw new ApiException(
                    "[{$e->getCode()}] {$e->getMessage()}",
                    $e->getCode(),
                    $e->getResponse() ? $e->getResponse()->getHeaders() : null,
                    $e->getResponse() ? $e->getResponse()->getBody()->getContents() : null, CommonUtil::get_masked_string(CommonUtil::wrapHttpRequest($request, $_tempBody, $formParams)), $masked_authentication_data
                ,$isTimedOut);
            }

            $statusCode = $response->getStatusCode();
			$logger->info('Receive Response from Service : HTTP Status Code :'.$statusCode);
            if ($statusCode < 200 || $statusCode > 299) {
                throw new ApiException(
                    sprintf(
                        '[%d] Error connecting to the API (%s)',
                        $statusCode,
                        $request->getUri()
                    ),
                    $statusCode,
                    $response->getHeaders(),
                    $response->getBody(), CommonUtil::get_masked_string(CommonUtil::wrapHttpRequest($request, $_tempBody, $formParams)), $masked_authentication_data
                );
            }

            $responseBody = $response->getBody();
            if ($returnType === '\SplFileObject') {
                $content = $responseBody; //stream goes to serializer
            } else {
                $content = $responseBody->getContents();
                if ($returnType !== 'string') {
                    $content = json_decode($content);
                }
            }

            return [
                ObjectSerializer::deserialize($content, $returnType, []),
                $response->getStatusCode(),
                $response->getHeaders()
            ];

        } catch (ApiException $e) {
            switch ($e->getCode()) {
                case 200:
                    $data = ObjectSerializer::deserialize(
                        json_decode($e->getResponseBody()),
                        '\Orbipay\PaymentsApi\Client\Model\PaymentResponse',
                        $e->getResponseHeaders()
                    );
                    $e->setResponseObject($data);
                    break;
                case 400:
                    $data = ObjectSerializer::deserialize(
                        json_decode($e->getResponseBody()),
                        '\Orbipay\PaymentsApi\Client\Model\ErrorResponseVo',
                        $e->getResponseHeaders()
                    );
                    $e->setResponseObject($data);
                    break;
                case 409:
                    $data = ObjectSerializer::deserialize(
                        json_decode($e->getResponseBody()),
                        '\Orbipay\PaymentsApi\Client\Model\ErrorResponseVo',
                        $e->getResponseHeaders()
                    );
                    $e->setResponseObject($data);
                    break;
                case 422:
                    $data = ObjectSerializer::deserialize(
                        json_decode($e->getResponseBody()),
                        '\Orbipay\PaymentsApi\Client\Model\ErrorResponseVo',
                        $e->getResponseHeaders()
                    );
                    $e->setResponseObject($data);
                    break;
            }
            throw $e;
        }
    }


    /**
     * Create request for operation 'updatePayment'
     *
     * @param  string $channel The channel through which the API is invoked. (required)
     * @param  string $client_key The unique identifier assigned by EBPP to the client. (required)
     * @param  string $product The product identifier corresponding to the API. (required)
     * @param  string $timestamp The timestamp for the moment when the API request is created. (required)
     * @param  string $idempotent_request_key The unique token that clients can generate and maintain in order to identify an API request. (required)
     * @param  string $requestor_type Type of the requestor of the API. (required)
     * @param  string $id_payment The unique identifier assigned by EBPP to the payment. (required)
     * @param  string $id_customer The unique identifier assigned by EBPP to the customer. (required)
     * @param  UpdatePaymentRequest $update_payment_request Details of the Payment to be updated. (required)
     * @param  string $requestor The identifier for the requestor of the API. (optional)
     * @param  string $x_opay_headers Intended for the future use. (optional)
     * @param  string $trace_id The unique reference that can be used for tracing and debugging an API call. (optional)
     *
     * @throws \InvalidArgumentException
     * @return array
     */
    protected function updatePaymentRequest($channel, $client_key, $product, $timestamp, $idempotent_request_key, $requestor_type, $id_payment, $id_customer, $update_payment_request, $requestor = null, $x_opay_headers = null, $trace_id = null)
    {
    
     $logger = LoggerFactory::getLogger(__CLASS__);
    
        // verify the required parameter 'id_payment' is set
        if ($id_payment === null || (is_array($id_payment) && count($id_payment) === 0) || strlen($id_payment) === 0) {
     		$logger->error('Missing the required parameter $id_payment');
           throw new \InvalidArgumentException(
                 'Missing the required parameter $id_payment'
             );
        }
        // verify the required parameter 'id_customer' is set
        if ($id_customer === null || (is_array($id_customer) && count($id_customer) === 0) || strlen($id_customer) === 0) {
     		$logger->error('Missing the required parameter $id_customer');
           throw new \InvalidArgumentException(
                 'Missing the required parameter $id_customer'
             );
        }
       
		$method = 'PUT';
        $resourcePath = '/customers/{ID_CUSTOMER}/payments/{ID_PAYMENT}';
        $formParams = [];
        $queryParams = [];
        $headerParams = [];
        $httpBody = '';
        $multipart = false;
        $masked_authentication_data = '';

        // header params
        if ($channel !== null) {
            $headerParams['channel'] = ObjectSerializer::toHeaderValue($channel);
        }
        // header params
        if ($client_key !== null) {
            $headerParams['client_key'] = ObjectSerializer::toHeaderValue($client_key);
        }
        // header params
        if ($product !== null) {
            $headerParams['product'] = ObjectSerializer::toHeaderValue($product);
        }
        // header params
        if ($timestamp !== null) {
            $headerParams['timestamp'] = ObjectSerializer::toHeaderValue($timestamp);
        }
        // header params
        if ($idempotent_request_key !== null) {
            $headerParams['idempotent_request_key'] = ObjectSerializer::toHeaderValue($idempotent_request_key);
        }
        // header params
        if ($requestor_type !== null) {
            $headerParams['requestor_type'] = ObjectSerializer::toHeaderValue($requestor_type);
        }
        // header params
        if ($requestor !== null) {
            $headerParams['requestor'] = ObjectSerializer::toHeaderValue($requestor);
        }
        // header params
        if ($x_opay_headers !== null) {
            $headerParams['X-OPAY-Headers'] = ObjectSerializer::toHeaderValue($x_opay_headers);
        }
        // header params
        if ($trace_id !== null) {
            $headerParams['trace_id'] = ObjectSerializer::toHeaderValue($trace_id);
        }

        // path params
        if ($id_payment !== null) {
            $resourcePath = str_replace(
                '{' . 'ID_PAYMENT' . '}',
                ObjectSerializer::toPathValue($id_payment),
                $resourcePath
            );
        }
        // path params
        if ($id_customer !== null) {
            $resourcePath = str_replace(
                '{' . 'ID_CUSTOMER' . '}',
                ObjectSerializer::toPathValue($id_customer),
                $resourcePath
            );
        }

        // body params
        $_tempBody = null;
        if (isset($update_payment_request)) {
            $_tempBody = $update_payment_request;
        }

        if ($multipart) {
            $headers = $this->headerSelector->selectHeadersForMultipart(
                ['application/json']
            );
        } else {
            $headers = $this->headerSelector->selectHeaders(
                ['application/json'],
                ['application/json']
            );
        }

        // for model (json/xml)
        if (isset($_tempBody)) {
            // $_tempBody is the method argument, if present
            $httpBody = $_tempBody;
            // \stdClass has no __toString(), so we should encode it manually
            if ($httpBody instanceof \stdClass && $headers['Content-Type'] === 'application/json') {
                $httpBody = \GuzzleHttp\json_encode($httpBody);
            }
        } elseif (count($formParams) > 0) {
            if ($multipart) {
                $multipartContents = [];
                foreach ($formParams as $formParamName => $formParamValue) {
                    $multipartContents[] = [
                        'name' => $formParamName,
                        'contents' => $formParamValue
                    ];
                }
                // for HTTP post (form)
                $httpBody = new MultipartStream($multipartContents);

            } elseif ($headers['Content-Type'] === 'application/json') {
                $httpBody = \GuzzleHttp\json_encode($formParams);

            } else {
                // for HTTP post (form)
                $httpBody = \GuzzleHttp\Psr7\build_query($formParams);
            }
        }
        
        if($logger->isDebugEnabled()){
            $logger->debug('HTTP Method     :'.$method);
            $logger->debug('HTTP URL        :'.$this->config->getHost() . $resourcePath);
        }
        
         if(empty($this->getConfig()->getClientApiKey())){
        	$this->getConfig()->setClientApiKey($this->getConfig()->getClientKey());
        }
        
        if(empty($this->getConfig()->getAuthScheme()) || $this->getConfig()->authScheme1 == $this->getConfig()->getAuthScheme()){
	         
	        list($hash, $masked_authentication_data) = AuthenticationUtil::computeOPAY1HMACSHA256Hash($method, $resourcePath, $headerParams, $queryParams, $formParams, $_tempBody, $this->getConfig()->getClientSignatureKey());
	        $headerParams['Authorization'] = $this->getConfig()->authScheme1 . " Credential=" . $this->getConfig()->getClientApiKey() . ",Signature=" . $hash;
        
        }elseif($this->getConfig()->authScheme2 == $this->getConfig()->getAuthScheme()){
				
	        $hash = AuthenticationUtil::computeOPAY2HMACSHA256Hash($this->getConfig()->getClientKey(), $this->getConfig()->getClientApiKey() ,$this->getConfig()->getClientSignatureKey(), $headerParams['timestamp']);
	        $headerParams['Authorization'] = $this->getConfig()->authScheme2 . " Credential=" . $this->getConfig()->getClientApiKey() . ",Signature=" . $hash;
	                
        }elseif($this->getConfig()->authScheme3 == $this->getConfig()->getAuthScheme()){
        	$headerParams['Authorization'] = $this->getConfig()->authScheme3 . " Credential=" . $this->getConfig()->getClientApiKey();
        }else{
        	$logger->error('Invalid auth_scheme received.');
           throw new \InvalidArgumentException('Invalid auth_scheme received.');
        }
        


        $defaultHeaders = [];
        if ($this->config->getUserAgent()) {
            $defaultHeaders['User-Agent'] = $this->config->getUserAgent();
        }

        $headers = array_merge(
            $defaultHeaders,
            $headerParams,
            $headers
        );

		if($logger->isDebugEnabled()){
			$logger->debug('Request Headers  :'.json_encode($headers));
		}
		
        $query = \GuzzleHttp\Psr7\build_query($queryParams);
        return [ new Request(
            $method,
            $this->config->getHost() . $resourcePath . ($query ? "?{$query}" : ''),
            $headers,
            $httpBody
        ), $_tempBody, $formParams, $masked_authentication_data];
    }

    /**
     * Create http client option
     *
     * @throws \RuntimeException on file opening failure
     * @return array of http client options
     */
    protected function createHttpClientOption()
    {
        $options = [];
        if ($this->config->getDebug()) {
            $options[RequestOptions::DEBUG] = fopen($this->config->getDebugFile(), 'a');
            if (!$options[RequestOptions::DEBUG]) {
                throw new \RuntimeException('Failed to open the debug file: ' . $this->config->getDebugFile());
            }
        }
        
        $timeoutValue = getenv("ORBIPAY_PAYMENTS_API_TIMEOUT_SECONDS");
        if (is_numeric($timeoutValue) && ((float)$timeoutValue) > 0) {
            $options[RequestOptions::TIMEOUT] = (float)$timeoutValue;
        } else {
        	$logger = LoggerFactory::getLogger(__CLASS__);
            $logger->warn('Environment Variable '."ORBIPAY_PAYMENTS_API_TIMEOUT_SECONDS".' was not set or value set is less than zero please configure it properly');
            $options[RequestOptions::TIMEOUT] = 60;
        }

        return $options;
    }
}
