<?php

namespace Orbipay\PaymentsApi\Client;

use Psr\Http\Message\RequestInterface;
use Orbipay\PaymentsApi\Client\Wrappers\InvocationContext;
use lf4php\LoggerFactory;

/**
 * Warning:
 *
 * <p><font color="red"> This class subject to change without prior notice, Please dont use this class directly. </font></p>


 *
 * @category Class
 * @package Orbipay\PaymentsApi\Client
 */


class CommonUtil
{

	 public static function isAPICallSuccessful(ApiException $e)
    {
        return ($e->getCode() === 200 || $e->getCode() === 201);
    }

    public static function getValidationMethod($className, $methodName)
    {
        return $className . '::' . $methodName;

    }

    public static function json_serialize($vars, $parent_vars = [])
    {
        foreach ($vars as &$value) {
            if (is_object($value) && method_exists($value, 'jsonSerialize')) {
                $value = $value->jsonSerialize();
            }
        }
        return array_merge($parent_vars, $vars);
    }

	/**
     * @param mixed $data
     * @return string
     */
    public static function get_masked_string($data)
    {
        return json_encode(self::mask_properties($data),JSON_UNESCAPED_UNICODE|JSON_UNESCAPED_SLASHES);
    }
    
    
	/**
     * @param mixed $data
     * @return mixed
     */
    public static function mask_properties($data)
    {
        if (is_array($data)) {
            return self::mask_array_properties($data);
        } else if (is_object($data)) {
            return self::mask_object_properties($data);
        } else {
            return $data;
        }
    }
    
	/**
     * @param $object
     * @return mixed
     */
    public static function mask_object_properties($object)
    {
        if (method_exists($object, 'jsonSerialize')) {
            return self::mask_array_properties($object->jsonSerialize());
        }

        $public = new \stdClass();
        if ($object INSTANCEOF \stdClass) {
            foreach ($object as $key => $value) {
                $sensitive_info = self::getSensitiveInfo();
                if(array_key_exists($key,$sensitive_info)){
                    if (is_array($value)) {
                        $public->$key = self::mask_entire_array_properties($value,$sensitive_info[$key]);
                    } else if (is_object($value)) {
                        $public->$key = self::mask_entire_object_properties($value,$sensitive_info[$key]);
                    } else {
                        $public->$key = self::mask_string($value, $sensitive_info[$key]);
                    }
                }
                else{
                    if (is_array($value)) {
                        $public->$key = self::mask_array_properties($value);
                    } else if (is_object($value)) {
                        $public->$key = self::mask_object_properties($value);
                    } else {
                        $sensitive_info = self::getSensitiveInfo();
                        if (array_key_exists($key, $sensitive_info)) {
                            $public->$key = self::mask_string($value, $sensitive_info[$key]);
                        } else {
                            $public->$key = $value;
                        }
                    }
                }
            }
        }
        return $public;
    }

    /**
     * @param $array
     * @return array
     */
    public static function mask_array_properties($array)
    {
        $public = [];
        if (self::isAssociativeArray($array)) {
            foreach ($array as $key => $value) {
                $sensitive_info = self::getSensitiveInfo();
                if(array_key_exists($key,$sensitive_info)){
                    if (is_array($value)) {
                        $public[$key] = self::mask_entire_array_properties($value,$sensitive_info[$key]);
                    } else if (is_object($value)) {
                        $public[$key] = self::mask_entire_object_properties($value,$sensitive_info[$key]);
                    } else {
                        $public[$key] = self::mask_string($value, $sensitive_info[$key]);
                    }
                }
                else{
                    if (is_array($value)) {
                        $public[$key] = self::mask_array_properties($value);
                    } else if (is_object($value)) {
                        $public[$key] = self::mask_object_properties($value);
                    } else {
                        $sensitive_info = self::getSensitiveInfo();
                        if (array_key_exists($key, $sensitive_info)) {
                            $public[$key] = self::mask_string($value, $sensitive_info[$key]);
                        } else {
                            $public[$key] = $value;
                        }
                    }
                }
            }
        } else {
            foreach ($array as $item) {
                if (is_object($item)) {
                    $public[] = self::mask_object_properties($item);
                } else if (is_array($item)) {
                    $public[] = self::mask_array_properties($item);
                } else {
                    $public[] = $item;
                }
            }
        }
        return $public;
    }


    /**
     * @param $array
     * @param
     * @return array
     */
    public static function mask_entire_array_properties($array,$visible_length = 0)
    {
        $public = [];
        if (self::isAssociativeArray($array)) {
            foreach ($array as $key => $value) {
                if (is_array($value)) {
                    $public[$key] = self::mask_entire_array_properties($value,$visible_length);
                } else if (is_object($value)) {
                    $public[$key] = self::mask_entire_object_properties($value,$visible_length);
                } else {
                        $public[$key] = self::mask_string($value, $visible_length);
                }
            }
        } else {
            foreach ($array as $item) {
                if (is_object($item)) {
                    $public[] = self::mask_entire_object_properties($item,$visible_length);
                } else if (is_array($item)) {
                    $public[] = self::mask_entire_array_properties($item,$visible_length);
                } else {
                    $public[] = $item;
                }
            }
        }
        return $public;
    }

    /**
     * @param $object
     * @return mixed
     */
    public static function mask_entire_object_properties($object,$visible_length =0)
    {
        if (method_exists($object, 'jsonSerialize')) {
            return self::mask_entire_array_properties($object->jsonSerialize(),$visible_length);
        }

        $public = new \stdClass();
        if ($object INSTANCEOF \stdClass) {
            foreach ($object as $key => $value) {
                if (is_array($value)) {
                    $public->$key = self::mask_entire_array_properties($value,$visible_length);
                } else if (is_object($value)) {
                    $public->$key = self::mask_entire_object_properties($value,$visible_length);
                } else {
                    $public->$key = self::mask_string($value, $visible_length);
                }
            }
        }
        return $public;
    }
    
    
    public static function mask_string($data, $visible_length = 0)
    {
        if (!is_string($data) || strlen($data) === 0) {
            return $data;
        }

        $string_length = strlen($data);

        if (!is_int($visible_length) || $visible_length <= 0 || ($string_length - $visible_length) <= 0) {
            return "**********";
        }

        return "**********" . substr($data, -$visible_length);
    }
    
    
	public static function getSensitiveInfo()
    {
        return [  'account_number' => 3 ,  'ssn' => 0 ,  'card_cvv_number' => 0 ,  'secret' => 0 ,  'expiry_date' => 0 ,  'custom_fields' => 2 ,  'customer_reference' => 3   ];
    }
    
    
    public static function getTimeStamp()
    {
        $utimestamp = microtime(true);
        $timestamp = floor($utimestamp);
        $milliseconds = round(($utimestamp - $timestamp) * 1000);
        return date(preg_replace('`(?<!\\\\)v`', $milliseconds, 'Y-m-d\ H:i:s.vP'), $timestamp);
    }
    
    public static function getTechDifficultiesError()
    {
    	$error_response = new \Orbipay\PaymentsApi\Client\Wrappers\ErrorResponse;
    	$errors = array();
        $error = new \Orbipay\PaymentsApi\Client\Wrappers\Error();
        $error->setCode('error_tech_difficulties');
        $error->setMessage('There is a problem processing this request due to technical difficulties.');
        $errors[] = $error;
        $error_response->setErrors($errors);
        return $error_response; 
    }
    
    public static function wrapHttpRequest(RequestInterface $http_request = null, $_tempBody = null, $formParams = null)
    {
        $request = array();
        if ($http_request != null) {
            $request['method'] = $http_request->getMethod();
            $request['uri'] = $http_request->getUri();
            $request['headers'] = $http_request->getHeaders();
            $request['http_protocol_version'] = $http_request->getProtocolVersion();
        }

        if (isset($_tempBody)) {
            $request['body'] = json_decode($_tempBody);
        } else {
            $request['body'] = AuthenticationUtil::getFormString(CommonUtil::mask_properties($formParams));
        }

        return $request;
    }
    
     public static function isAssociativeArray($arr)
    {
        if (!is_array($arr)) return false;

        if (array() === $arr) return true;
        return array_keys($arr) !== range(0, count($arr) - 1) ? true : false;
    }
    
     /**
     * @return InvocationContex
     */
    public static function getInvocationContext(array $headers = null)
    {
        /**
         * @var InvocationContext
         */
        $invocationContext = null;

        if ($headers !== null) {
            $invocationContext = new InvocationContext();
			
            if (isset($headers['product']) && count($headers['product']) > 0) {
                $invocationContext->setProduct(array_values($headers['product'])[0]);
            }
            if (isset($headers['trace_id']) && count($headers['trace_id']) > 0) {
                $invocationContext->setTraceId(array_values($headers['trace_id'])[0]);
            }
            if (isset($headers['idempotent_request_key']) && count($headers['idempotent_request_key']) > 0) {
                $invocationContext->setIdempotentRequestKey(array_values($headers['idempotent_request_key'])[0]);
            }
            if (isset($headers['request_uuid']) && count($headers['request_uuid']) > 0) {
                $invocationContext->setRequestUuid(array_values($headers['request_uuid'])[0]);
            }
            if (isset($headers['channel']) && count($headers['channel']) > 0) {
                $invocationContext->setChannel(array_values($headers['channel'])[0]);
            }
            if (isset($headers['secret']) && count($headers['secret']) > 0) {
                $invocationContext->setSecret(array_values($headers['secret'])[0]);
            }
            if (isset($headers['requestor']) && count($headers['requestor']) > 0) {
                $invocationContext->setRequestor(array_values($headers['requestor'])[0]);
            }
            if (isset($headers['client_key']) && count($headers['client_key']) > 0) {
                $invocationContext->setClientKey(array_values($headers['client_key'])[0]);
            }
            if (isset($headers['X-OPAY-Headers']) && count($headers['X-OPAY-Headers']) > 0) {
                $invocationContext->setXOpayHeaders(array_values($headers['X-OPAY-Headers'])[0]);
            }
            if (isset($headers['requestor_type']) && count($headers['requestor_type']) > 0) {
                $invocationContext->setRequestorType(array_values($headers['requestor_type'])[0]);
            }
            if (isset($headers['api_key']) && count($headers['api_key']) > 0) {
                $invocationContext->setApiKey(array_values($headers['api_key'])[0]);
            }
            if (isset($headers['response_codes']) && count($headers['response_codes']) > 0) {
                $invocationContext->setResponseCodes(array_values($headers['response_codes'])[0]);
            }
            if (isset($headers['response_text']) && count($headers['response_text']) > 0) {
                $invocationContext->setResponseText(array_values($headers['response_text'])[0]);
            }
            if (isset($headers['auth_scheme']) && count($headers['auth_scheme']) > 0) {
                $invocationContext->setAuthScheme(array_values($headers['auth_scheme'])[0]);
            }
            if (isset($headers['timestamp']) && count($headers['timestamp']) > 0) {
                $invocationContext->setTimestamp(array_values($headers['timestamp'])[0]);
            }
            
            $logger = LoggerFactory::getLogger(__CLASS__);
            $logger->info('request_UUID :'.$invocationContext->getRequestUuid());
        }

        return $invocationContext;
    }
    
    public static function isJson($string)
    {
        json_decode($string);
        return (json_last_error() == JSON_ERROR_NONE);
    }
    
}
