<?php



namespace Orbipay\PaymentsApi\Client\Handlers;

use lf4php\LoggerFactory;
use Orbipay\PaymentsApi\Client\CommonUtil;
use Orbipay\PaymentsApi\Client\Configuration;
use Orbipay\PaymentsApi\Client\ApiException;
use Orbipay\PaymentsApi\Client\OrbipayApiException;
use Orbipay\PaymentsApi\Client\ObjectSerializer;
use Orbipay\PaymentsApi\Client\Wrappers\CustomerAccounts;
use Orbipay\PaymentsApi\Client\Api\CustomerAccountApi;

 
/**
 * Warning:
 *
 * <p><font color="red"> This class subject to change without prior notice, Please dont use this class directly. </font></p>


 *
 * @category Class
 * @package  Orbipay\PaymentsApi\Client\Handlers
 */
class  RetrieveCustomerAccountsHandler
{ 
    /**
     * @param CustomerAccounts $customerAccounts
     * @return CustomerAccounts
     * @throws \Orbipay\PaymentsApi\Client\OrbipayApiException
     */
    public function process(CustomerAccounts $customerAccounts)
    {
        return $this->callAPI($customerAccounts);
    }
    
     /**
     * @param CustomerAccounts $customerAccounts
     * @return CustomerAccounts
     * @throws \Orbipay\PaymentsApi\Client\OrbipayApiException
     */
    private function callAPI(CustomerAccounts $customerAccounts)
    {
        $configuration = new Configuration();
        $configuration->setHost($customerAccounts->getApiEndPoint());
        $configuration->setClientKey($customerAccounts->getInvocationContext()->getClientKey());
        $configuration->setClientSignatureKey($customerAccounts->getInvocationContext()->getSecret());
        $configuration->setClientApiKey($customerAccounts->getInvocationContext()->getAPIKey());
        $configuration->setAuthScheme($customerAccounts->getInvocationContext()->getAuthScheme());
        

        $apiInstance = new CustomerAccountApi(null, $configuration, null);
        /**
         * @var array of Orbipay\PaymentsApi\Client\Model\CustomerAccountsResponse, HTTP status code, HTTP response headers (array of strings)
         */
        $response = null;
	
		 try {
		 
		 	
			
			$channel = $customerAccounts->GetInvocationContext()->GetChannel();
		 	
			
			$client_key = $customerAccounts->GetInvocationContext()->GetClientKey();
		 	
			
			$product = $customerAccounts->GetInvocationContext()->GetProduct();
		 	
			
			$timestamp = $customerAccounts->GetInvocationContext()->GetTimestamp();
			if($timestamp == null){ 
				$timestamp = CommonUtil::getTimeStamp();
			}
		 	
			
			$idempotent_request_key = $customerAccounts->GetInvocationContext()->GetIdempotentRequestKey();
		 	
			
			$requestor_type = $customerAccounts->GetInvocationContext()->GetRequestorType();
		 	
			
			$id_customer = $customerAccounts->GetIdCustomer();
		 	
			
			$requestor = $customerAccounts->GetInvocationContext()->GetRequestor();
		 	
			
			$x_opay_headers = $customerAccounts->GetInvocationContext()->GetXOpayHeaders();
		 	
			
			$trace_id = $customerAccounts->GetInvocationContext()->GetTraceId();
		 	
			
			$page_size = $customerAccounts->GetPageSize();
		 	
			
			$account_number = $customerAccounts->GetAccountNumber();
		 	
			
			$status = $customerAccounts->GetStatus();

		 
		 
		    $response = $apiInstance->retrieveCustomerAccounts( $channel,  $client_key,  $product,  $timestamp,  $idempotent_request_key,  $requestor_type,  $id_customer,  $requestor,  $x_opay_headers,  $trace_id,  $page_size,  $account_number,  $status);
		    
		} catch (GuzzleException $e) {
		    throw new OrbipayApiException($e->getMessage(), $e->getCode(), $e);
		} catch (ApiException $e) {
		    throw OrbipayApiException::constructFromApiException($e);
		} catch (\Exception $e) {
		    throw new OrbipayApiException($e->getMessage(), $e->getCode(), $e);
		}
		
		return $this->wrapResponse($response);

	}
	

	/**
     * @param null $apiResponse
     * @return CustomerAccounts
     */
    private function wrapResponse($apiResponse = null)
    {
    	 $wrapperResponse = null;
    	 
        if ($apiResponse === null)
            return $wrapperResponse;

        list($response, $statusCode, $headers) = $apiResponse;
        
        $logger = LoggerFactory::getLogger(__CLASS__);
        if($logger->isDebugEnabled()){
            $logger->debug('HTTP Status Code :'.$statusCode);
            $logger->debug('Response Headers  :'.json_encode($headers));
            $logger->debug('HTTP Response Body      :'.CommonUtil::get_masked_string(json_decode($response)));
        }
	$wrapperResponse = ObjectSerializer::deserialize(json_decode($response),'\Orbipay\PaymentsApi\Client\Wrappers\CustomerAccounts',null);
	if($wrapperResponse === null){
		$wrapperResponse = new \Orbipay\PaymentsApi\Client\Wrappers\CustomerAccounts();
	}
        $wrapperResponse->setInvocationContext(CommonUtil::getInvocationContext($headers));
		$wrapperResponse->setHttpStatusCode($statusCode);
        return $wrapperResponse;
    }
	
}
