<?php



namespace Orbipay\PaymentsApi\Client\Handlers;

use lf4php\LoggerFactory;
use Orbipay\PaymentsApi\Client\CommonUtil;
use Orbipay\PaymentsApi\Client\Configuration;
use Orbipay\PaymentsApi\Client\ApiException;
use Orbipay\PaymentsApi\Client\OrbipayApiException;
use Orbipay\PaymentsApi\Client\ObjectSerializer;
use Orbipay\PaymentsApi\Client\Wrappers\FundingAccounts;
use Orbipay\PaymentsApi\Client\Api\FundingAccountApi;

 
/**
 * Warning:
 *
 * <p><font color="red"> This class subject to change without prior notice, Please dont use this class directly. </font></p>


 *
 * @category Class
 * @package  Orbipay\PaymentsApi\Client\Handlers
 */
class  RetrieveFundingAccountsPageHandler
{ 
    /**
     * @param FundingAccounts $fundingAccounts
     * @return FundingAccounts
     * @throws \Orbipay\PaymentsApi\Client\OrbipayApiException
     */
    public function process(FundingAccounts $fundingAccounts)
    {
        return $this->callAPI($fundingAccounts);
    }
    
     /**
     * @param FundingAccounts $fundingAccounts
     * @return FundingAccounts
     * @throws \Orbipay\PaymentsApi\Client\OrbipayApiException
     */
    private function callAPI(FundingAccounts $fundingAccounts)
    {
        $configuration = new Configuration();
        $configuration->setHost($fundingAccounts->getApiEndPoint());
        $configuration->setClientKey($fundingAccounts->getInvocationContext()->getClientKey());
        $configuration->setClientSignatureKey($fundingAccounts->getInvocationContext()->getSecret());
        $configuration->setClientApiKey($fundingAccounts->getInvocationContext()->getAPIKey());
        $configuration->setAuthScheme($fundingAccounts->getInvocationContext()->getAuthScheme());
        

        $apiInstance = new FundingAccountApi(null, $configuration, null);
        /**
         * @var array of Orbipay\PaymentsApi\Client\Model\FundingAccountsResponse, HTTP status code, HTTP response headers (array of strings)
         */
        $response = null;
	
		 try {
		 
		 	
			
			$channel = $fundingAccounts->GetInvocationContext()->GetChannel();
		 	
			
			$client_key = $fundingAccounts->GetInvocationContext()->GetClientKey();
		 	
			
			$product = $fundingAccounts->GetInvocationContext()->GetProduct();
		 	
			
			$timestamp = $fundingAccounts->GetInvocationContext()->GetTimestamp();
			if($timestamp == null){ 
				$timestamp = CommonUtil::getTimeStamp();
			}
		 	
			
			$idempotent_request_key = $fundingAccounts->GetInvocationContext()->GetIdempotentRequestKey();
		 	
			
			$requestor_type = $fundingAccounts->GetInvocationContext()->GetRequestorType();
		 	
			
			$id_customer = $fundingAccounts->GetIdCustomer();
		 	
			
			$query_id = $fundingAccounts->GetQueryId();
		 	
			
			$requestor = $fundingAccounts->GetInvocationContext()->GetRequestor();
		 	
			
			$x_opay_headers = $fundingAccounts->GetInvocationContext()->GetXOpayHeaders();
		 	
			
			$trace_id = $fundingAccounts->GetInvocationContext()->GetTraceId();
		 	
			
			$page_size = $fundingAccounts->GetPageSize();
		 	
			
			$from_index = $fundingAccounts->GetFromIndex();
		 	
			
			$to_index = $fundingAccounts->GetToIndex();
		 
		 
		    $response = $apiInstance->retrieveFundingAccountsPage( $channel,  $client_key,  $product,  $timestamp,  $idempotent_request_key,  $requestor_type,  $id_customer,  $query_id,  $requestor,  $x_opay_headers,  $trace_id,  $page_size,  $from_index,  $to_index);
		    
		} catch (GuzzleException $e) {
		    throw new OrbipayApiException($e->getMessage(), $e->getCode(), $e);
		} catch (ApiException $e) {
		    throw OrbipayApiException::constructFromApiException($e);
		} catch (\Exception $e) {
		    throw new OrbipayApiException($e->getMessage(), $e->getCode(), $e);
		}
		
		return $this->wrapResponse($response);

	}
	

	/**
     * @param null $apiResponse
     * @return FundingAccounts
     */
    private function wrapResponse($apiResponse = null)
    {
    	 $wrapperResponse = null;
    	 
        if ($apiResponse === null)
            return $wrapperResponse;

        list($response, $statusCode, $headers) = $apiResponse;
        
        $logger = LoggerFactory::getLogger(__CLASS__);
        if($logger->isDebugEnabled()){
            $logger->debug('HTTP Status Code :'.$statusCode);
            $logger->debug('Response Headers  :'.json_encode($headers));
            $logger->debug('HTTP Response Body      :'.CommonUtil::get_masked_string(json_decode($response)));
        }
	$wrapperResponse = ObjectSerializer::deserialize(json_decode($response),'\Orbipay\PaymentsApi\Client\Wrappers\FundingAccounts',null);
	if($wrapperResponse === null){
		$wrapperResponse = new \Orbipay\PaymentsApi\Client\Wrappers\FundingAccounts();
	}
        $wrapperResponse->setInvocationContext(CommonUtil::getInvocationContext($headers));
		$wrapperResponse->setHttpStatusCode($statusCode);
        return $wrapperResponse;
    }
	
}
