<?php



namespace Orbipay\PaymentsApi\Client\Handlers;

use lf4php\LoggerFactory;
use Orbipay\PaymentsApi\Client\CommonUtil;
use Orbipay\PaymentsApi\Client\Configuration;
use Orbipay\PaymentsApi\Client\ApiException;
use Orbipay\PaymentsApi\Client\OrbipayApiException;
use Orbipay\PaymentsApi\Client\ObjectSerializer;
use Orbipay\PaymentsApi\Client\Wrappers\FundingAccount;
use Orbipay\PaymentsApi\Client\Api\FundingAccountApi;

 
/**
 * Warning:
 *
 * <p><font color="red"> This class subject to change without prior notice, Please dont use this class directly. </font></p>


 *
 * @category Class
 * @package  Orbipay\PaymentsApi\Client\Handlers
 */
class  UpdateFundingAccountHandler
{ 
    /**
     * @param FundingAccount $fundingAccount
     * @return FundingAccount
     * @throws \Orbipay\PaymentsApi\Client\OrbipayApiException
     */
    public function process(FundingAccount $fundingAccount)
    {
        return $this->callAPI($fundingAccount);
    }
    
     /**
     * @param FundingAccount $fundingAccount
     * @return FundingAccount
     * @throws \Orbipay\PaymentsApi\Client\OrbipayApiException
     */
    private function callAPI(FundingAccount $fundingAccount)
    {
        $configuration = new Configuration();
        $configuration->setHost($fundingAccount->getApiEndPoint());
        $configuration->setClientKey($fundingAccount->getInvocationContext()->getClientKey());
        $configuration->setClientSignatureKey($fundingAccount->getInvocationContext()->getSecret());
        $configuration->setClientApiKey($fundingAccount->getInvocationContext()->getAPIKey());
        $configuration->setAuthScheme($fundingAccount->getInvocationContext()->getAuthScheme());
        

        $apiInstance = new FundingAccountApi(null, $configuration, null);
        /**
         * @var array of Orbipay\PaymentsApi\Client\Model\FundingAccountResponse, HTTP status code, HTTP response headers (array of strings)
         */
        $response = null;
	
		 try {
		 
		 	
			
			$channel = $fundingAccount->GetInvocationContext()->GetChannel();
		 	
			
			$client_key = $fundingAccount->GetInvocationContext()->GetClientKey();
		 	
			
			$product = $fundingAccount->GetInvocationContext()->GetProduct();
		 	
			
			$timestamp = $fundingAccount->GetInvocationContext()->GetTimestamp();
			if($timestamp == null){ 
				$timestamp = CommonUtil::getTimeStamp();
			}
		 	
			
			$idempotent_request_key = $fundingAccount->GetInvocationContext()->GetIdempotentRequestKey();
		 	
			
			$requestor_type = $fundingAccount->GetInvocationContext()->GetRequestorType();
		 	
			
			$id_funding_account = $fundingAccount->GetIdFundingAccount();
		 	
			
			$id_customer = $fundingAccount->GetIdCustomer();
		 	
			$update_funding_account_request = $this->populateBody($fundingAccount);
			
		 	
			
			$requestor = $fundingAccount->GetInvocationContext()->GetRequestor();
		 	
			
			$x_opay_headers = $fundingAccount->GetInvocationContext()->GetXOpayHeaders();
		 	
			
			$trace_id = $fundingAccount->GetInvocationContext()->GetTraceId();
		 
		 
		    $response = $apiInstance->updateFundingAccount( $channel,  $client_key,  $product,  $timestamp,  $idempotent_request_key,  $requestor_type,  $id_funding_account,  $id_customer,  $update_funding_account_request,  $requestor,  $x_opay_headers,  $trace_id);
		    
		} catch (GuzzleException $e) {
		    throw new OrbipayApiException($e->getMessage(), $e->getCode(), $e);
		} catch (ApiException $e) {
		    throw OrbipayApiException::constructFromApiException($e);
		} catch (\Exception $e) {
		    throw new OrbipayApiException($e->getMessage(), $e->getCode(), $e);
		}
		
		return $this->wrapResponse($response);

	}
	
	    
     /**
     * @param FundingAccount $fundingAccount
     * @return \Orbipay\PaymentsApi\Client\Model\UpdateFundingAccountRequest
     */
	private function populateBody(FundingAccount $fundingAccount)
    {

        $serializedObject=ObjectSerializer::sanitizeForSerialization($fundingAccount);

        $encodedString = json_encode($serializedObject);

		$jsonObj = json_decode($encodedString);

		

        $jsonEncode = json_encode($jsonObj);

        $requestBody = ObjectSerializer::deserialize(
            json_decode($jsonEncode),
            'Orbipay\PaymentsApi\Client\Model\UpdateFundingAccountRequest',
            null
        );
        
        return $requestBody;
    }

	/**
     * @param null $apiResponse
     * @return FundingAccount
     */
    private function wrapResponse($apiResponse = null)
    {
    	 $wrapperResponse = null;
    	 
        if ($apiResponse === null)
            return $wrapperResponse;

        list($response, $statusCode, $headers) = $apiResponse;
        
        $logger = LoggerFactory::getLogger(__CLASS__);
        if($logger->isDebugEnabled()){
            $logger->debug('HTTP Status Code :'.$statusCode);
            $logger->debug('Response Headers  :'.json_encode($headers));
            $logger->debug('HTTP Response Body      :'.CommonUtil::get_masked_string(json_decode($response)));
        }
	$wrapperResponse = ObjectSerializer::deserialize(json_decode($response),'\Orbipay\PaymentsApi\Client\Wrappers\FundingAccount',null);
	if($wrapperResponse === null){
		$wrapperResponse = new \Orbipay\PaymentsApi\Client\Wrappers\FundingAccount();
	}
        $wrapperResponse->setInvocationContext(CommonUtil::getInvocationContext($headers));
		$wrapperResponse->setHttpStatusCode($statusCode);
        return $wrapperResponse;
    }
	
}
