<?php


namespace Orbipay\PaymentsApi\Client\Wrappers;

use \ArrayAccess;
use \Orbipay\PaymentsApi\Client\ObjectSerializer;
use \Orbipay\PaymentsApi\Client\CommonUtil;
use Orbipay\PaymentsApi\Client\OrbipayApiException;
use lf4php\LoggerFactory;

/**
 * Base Class Doc Comment
 *
 * @category Class
 * @package  Orbipay\PaymentsApi\Client\Wrappers
 */
class Base 
{

    /**
     * @var string
     */
    private $live_mode;

    /**
     * @var InvocationContext
     */
    private $invocation_context;

    /**
     * @var string
     */
    protected $api_end_point;

	private $http_status_code;
  
  	public function getHttpStatusCode()
  	{
  		return $this->http_status_code;
  	}
  	
  	/**
     * @internal
     * @param  $http_status_code
     */
    public function setHttpStatusCode($http_status_code)
    {
        $this->http_status_code = $http_status_code;
    }
  	

    /**
     * @internal
     * @return string
     */
    public function getLiveMode()
    {
        return $this->live_mode;
    }

    /**
     * @internal
     * @param string $live_mode
     */
    public function setLiveMode($live_mode)
    {
        $this->live_mode = $live_mode;
    }

    /**
     * @return InvocationContext
     */
    public function getInvocationContext()
    {
        return $this->invocation_context;
    }

    /**
     * @param InvocationContext $invocation_context
     */
    public function setInvocationContext(InvocationContext $invocation_context = null)
    {
        $this->invocation_context = $invocation_context;
    }

    /**
     * @return string
     */
    public function getApiEndPoint()
    {
        return $this->api_end_point;
    }

	protected function videChannel($channel)
	{
		$invocationContext = $this->getInvocationContext();
	    if (!isset($invocationContext)) {
	        $invocationContext = new InvocationContext();
	    }
		 $invocationContext->setChannel($channel);
		$this->setInvocationContext($invocationContext);
	}
	protected function requestedBy($requestor, $requestor_type)
	{
		$invocationContext = $this->getInvocationContext();
	    if (!isset($invocationContext)) {
	        $invocationContext = new InvocationContext();
	    }
		 $invocationContext->setRequestor($requestor);
		 $invocationContext->setRequestorType($requestor_type);
		$this->setInvocationContext($invocationContext);
	}
	protected function forClient($client_key)
	{
		$invocationContext = $this->getInvocationContext();
	    if (!isset($invocationContext)) {
	        $invocationContext = new InvocationContext();
	    }
		 $invocationContext->setClientKey($client_key);
		$this->setInvocationContext($invocationContext);
	}


    /**
     * @param string $live_mode
     * @param string $apiEndPoint
     */
    protected function setAPIEndpoint($live_mode = null, $apiEndPoint = null)
    {
    		$this->live_mode = $live_mode;
	    	
	    	if( $this->live_mode === "true"){
				$this->api_end_point = "https://api.orbipay.com/payments/v1";		
			}else if($this->live_mode ===("false") && empty($apiEndPoint)){
	    	    $this->api_end_point = "https://sbapi.orbipay.com/payments/v1";
			}else if($this->live_mode ===("false") && !empty($apiEndPoint)){
				$this->api_end_point = $apiEndPoint;	
			}else{
                $logger = LoggerFactory::getLogger(__CLASS__);
                $logger->error('Invalid value for live_mode');
                throw new OrbipayApiException('Invalid value for live_mode,  value Received in request : '.$this->live_mode.', valid values for live_mode: true, false');
			}
    }

    /**
     * @param InvocationContext $request
     * @param string $live_mode
     * @param string $apiEndPoint
     */
    public function withContext(InvocationContext $request, $live_mode = null, $apiEndPoint = null)
    {
        $this->setAPIEndpoint($live_mode, $apiEndPoint);

        if (!isset($this->invocation_context)) {
            $this->invocation_context = new InvocationContext();
        }

        if (isset($request)) {
	    $this->invocation_context->setProduct($request->getProduct());
	    $this->invocation_context->setSecret($request->getSecret());
	    $this->invocation_context->setIdempotentRequestKey($request->getIdempotentRequestKey());
	    $this->invocation_context->setXOpayHeaders($request->getXOpayHeaders());
	    $this->invocation_context->setApiKey($request->getApiKey());
	    $this->invocation_context->setAuthScheme($request->getAuthScheme());
	    $this->invocation_context->setTraceId($request->getTraceId());
        }
        
		$logger = LoggerFactory::getLogger(__CLASS__);
        $logger->info('Idempotent Request Key : '.$this->invocation_context->getIdempotentRequestKey());
    }


    /**
     * Gets the string presentation of the object
     *
     * @return string
     */
    public function __toString()
    {
        if (defined('JSON_PRETTY_PRINT')) { // use JSON pretty print
            return json_encode(
                ObjectSerializer::sanitizeForSerialization($this),
                JSON_PRETTY_PRINT
            );
        }

        return json_encode(ObjectSerializer::sanitizeForSerialization($this));
    }
    
     /**
     * Specify data which should be serialized to JSON
     * @link http://php.net/manual/en/jsonserializable.jsonserialize.php
     * @return mixed data which can be serialized by <b>json_encode</b>,
     * which is a value of any type other than a resource.
     * @since 5.4.0
     */
    public function jsonSerialize()
    {
        return CommonUtil::json_serialize(get_object_vars($this));
    }

}
