<?php

/**
 * Customer
 *
 * PHP version 5
 *
 * @category Class
 * @package  Orbipay\PaymentsApi\Client\Wrappers
 */



/**
 * Do not edit the class manually.
 */

namespace Orbipay\PaymentsApi\Client\Wrappers;

use \Orbipay\PaymentsApi\Client\ObjectSerializer;
use \Orbipay\PaymentsApi\Client\MultiMap;
use \Orbipay\PaymentsApi\Client\CommonUtil;
use Orbipay\PaymentsApi\Client\OrbipayApiException;
use lf4php\LoggerFactory;


/**
 * Customer Class Doc Comment
 * @category Class
 * @package  Orbipay\PaymentsApi\Client\Wrappers
 */
class Customer  extends Base 
{

	const DISCRIMINATOR = null;
    
        /**
      * Array of property to type mappings. Used for (de)serialization
      *
      * @var string[]
      */
    protected static $swaggerTypes = [
        'first_name' => 'string', 
        'last_name' => 'string', 
        'middle_name' => 'string', 
        'gender' => 'string', 
        'date_of_birth' => 'string', 
        'ssn' => 'string', 
        'locale' => 'string', 
        'email' => 'string', 
        'registered_email' => 'string', 
        'home_phone' => 'string', 
        'work_phone' => 'string', 
        'mobile_phone' => 'string', 
        'address' => '\Orbipay\PaymentsApi\Client\Wrappers\Address', 
        'custom_fields' => 'map[string,string]', 
        'customer_reference' => 'string', 
        'id' => 'string', 
        'status' => 'string', 
        'url' => 'string', 
        'comments' => 'string', 
        'customer_accounts' => '\Orbipay\PaymentsApi\Client\Wrappers\CustomerAccounts', 
        'funding_accounts' => '\Orbipay\PaymentsApi\Client\Wrappers\FundingAccounts', 
        'payments' => '\Orbipay\PaymentsApi\Client\Wrappers\Payments', 
        'payment_setups' => '\Orbipay\PaymentsApi\Client\Wrappers\PaymentSetups', 
        'audit_info' => '\Orbipay\PaymentsApi\Client\Wrappers\AuditInfo', 
        'id_customer' => 'string'
    ];

    /**
      * Array of property to format mappings. Used for (de)serialization
      *
      * @var string[]
      */
    protected static $swaggerFormats = [
        'first_name' => null,
        'last_name' => null,
        'middle_name' => null,
        'gender' => null,
        'date_of_birth' => null,
        'ssn' => null,
        'locale' => null,
        'email' => 'email',
        'registered_email' => 'email',
        'home_phone' => null,
        'work_phone' => null,
        'mobile_phone' => null,
        'address' => null,
        'custom_fields' => null,
        'customer_reference' => null,
        'id' => null,
        'status' => null,
        'url' => null,
        'comments' => null,
        'customer_accounts' => null,
        'funding_accounts' => null,
        'payments' => null,
        'payment_setups' => null,
        'audit_info' => null,
        'id_customer' => null
    ];

    /**
     * Array of property to type mappings. Used for (de)serialization
     *
     * @return array
     */
    public static function swaggerTypes()
    {
        return self::$swaggerTypes;
    }

    /**
     * Array of property to format mappings. Used for (de)serialization
     *
     * @return array
     */
    public static function swaggerFormats()
    {
        return self::$swaggerFormats;
    }

    /**
     * Array of attributes where the key is the local name,
     * and the value is the original name
     *
     * @var string[]
     */
    protected static $attributeMap = [
        'first_name' => 'first_name',
        'last_name' => 'last_name',
        'middle_name' => 'middle_name',
        'gender' => 'gender',
        'date_of_birth' => 'date_of_birth',
        'ssn' => 'ssn',
        'locale' => 'locale',
        'email' => 'email',
        'registered_email' => 'registered_email',
        'home_phone' => 'home_phone',
        'work_phone' => 'work_phone',
        'mobile_phone' => 'mobile_phone',
        'address' => 'address',
        'custom_fields' => 'custom_fields',
        'customer_reference' => 'customer_reference',
        'id' => 'id',
        'status' => 'status',
        'url' => 'url',
        'comments' => 'comments',
        'customer_accounts' => 'customer_accounts',
        'funding_accounts' => 'funding_accounts',
        'payments' => 'payments',
        'payment_setups' => 'payment_setups',
        'audit_info' => 'audit_info',
        'id_customer' => 'ID_CUSTOMER'
    ];
    
     /**
     * Array of attributes to setter functions (for deserialization of responses)
     *
     * @var string[]
     */
    protected static $setters = [
        'first_name' => 'setFirstName',
        'last_name' => 'setLastName',
        'middle_name' => 'setMiddleName',
        'gender' => 'setGender',
        'date_of_birth' => 'setDateOfBirth',
        'ssn' => 'setSsn',
        'locale' => 'setLocale',
        'email' => 'setEmail',
        'registered_email' => 'setRegisteredEmail',
        'home_phone' => 'setHomePhone',
        'work_phone' => 'setWorkPhone',
        'mobile_phone' => 'setMobilePhone',
        'address' => 'setAddress',
        'custom_fields' => 'setCustomFields',
        'customer_reference' => 'setCustomerReference',
        'id' => 'setId',
        'status' => 'setStatus',
        'url' => 'setUrl',
        'comments' => 'setComments',
        'customer_accounts' => 'setCustomerAccounts',
        'funding_accounts' => 'setFundingAccounts',
        'payments' => 'setPayments',
        'payment_setups' => 'setPaymentSetups',
        'audit_info' => 'setAuditInfo',
        'id_customer' => 'setIdCustomer'
    ];
    
     /**
     * Array of attributes to getter functions (for serialization of requests)
     *
     * @var string[]
     */
    protected static $getters = [
        'first_name' => 'getFirstName',
        'last_name' => 'getLastName',
        'middle_name' => 'getMiddleName',
        'gender' => 'getGender',
        'date_of_birth' => 'getDateOfBirth',
        'ssn' => 'getSsn',
        'locale' => 'getLocale',
        'email' => 'getEmail',
        'registered_email' => 'getRegisteredEmail',
        'home_phone' => 'getHomePhone',
        'work_phone' => 'getWorkPhone',
        'mobile_phone' => 'getMobilePhone',
        'address' => 'getAddress',
        'custom_fields' => 'getCustomFields',
        'customer_reference' => 'getCustomerReference',
        'id' => 'getId',
        'status' => 'getStatus',
        'url' => 'getUrl',
        'comments' => 'getComments',
        'customer_accounts' => 'getCustomerAccounts',
        'funding_accounts' => 'getFundingAccounts',
        'payments' => 'getPayments',
        'payment_setups' => 'getPaymentSetups',
        'audit_info' => 'getAuditInfo',
        'id_customer' => 'getIdCustomer'
    ];
    
      /**
     * Array of attributes where the key is the local name,
     * and the value is the original name
     *
     * @return array
     */
    public static function attributeMap()
    {
        return self::$attributeMap;
    }

    /**
     * Array of attributes to setter functions (for deserialization of responses)
     *
     * @return array
     */
    public static function setters()
    {
        return self::$setters;
    }
    
     /**
     * Array of attributes to getter functions (for serialization of requests)
     *
     * @return array
     */
    public static function getters()
    {
        return self::$getters;
    }
    

    /**
     * @internal
     * Sets first_name
     * @param string $first_name The first name of the customer. Either first_name or last_name is required to create a customer.
     * @return $this
     */
    public function setFirstName($first_name)
    {
        $this->first_name = $first_name;

        return $this;
    }

    /**
     * @internal
     * Sets last_name
     * @param string $last_name The last name of the customer. Either first_name or last_name is required to create a customer.
     * @return $this
     */
    public function setLastName($last_name)
    {
        $this->last_name = $last_name;

        return $this;
    }

    /**
     * @internal
     * Sets middle_name
     * @param string $middle_name The middle name of the customer.
     * @return $this
     */
    public function setMiddleName($middle_name)
    {
        $this->middle_name = $middle_name;

        return $this;
    }

    /**
     * @internal
     * Sets gender
     * @param string $gender The gender of the customer.
     * @return $this
     */
    public function setGender($gender)
    {
        $this->gender = $gender;

        return $this;
    }

    /**
     * @internal
     * Sets date_of_birth
     * @param string $date_of_birth The date of birth of the customer. It should be in the full-date format as per ISO8601, namely, YYYY-MM-DD.
     * @return $this
     */
    public function setDateOfBirth($date_of_birth)
    {
        $this->date_of_birth = $date_of_birth;

        return $this;
    }

    /**
     * @internal
     * Sets ssn
     * @param string $ssn The SSN of the customer if the account holder is an individual or the tax ID, if the customer is a business.
     * @return $this
     */
    public function setSsn($ssn)
    {
        $this->ssn = $ssn;

        return $this;
    }

    /**
     * @internal
     * Sets locale
     * @param string $locale The language in which the customer wants the alerts and notifications from EBPP. This is to be specified in the format,<br><i>< <b>ISO-639-1 language code</b>  >_< <b>ISO ALPHA-2 Country Code</b> ></i>.<br> For e.g., <b>en_US</b> indicates the language preference as US English, which is also the default value.
     * @return $this
     */
    public function setLocale($locale)
    {
        $this->locale = $locale;

        return $this;
    }

    /**
     * @internal
     * Sets email
     * @param string $email The email address of the customer to which notifications from EBPP will be sent. This will default to the registered_email while creating a customer.
     * @return $this
     */
    public function setEmail($email)
    {
        $this->email = $email;

        return $this;
    }

    /**
     * @internal
     * Sets registered_email
     * @param string $registered_email The email address of the customer registered with the client. This may be used to uniquely identify the customer if the client is set up accordingly in EBPP.
     * @return $this
     */
    public function setRegisteredEmail($registered_email)
    {
        $this->registered_email = $registered_email;

        return $this;
    }

    /**
     * @internal
     * Sets home_phone
     * @param string $home_phone The home phone number of the customer.
     * @return $this
     */
    public function setHomePhone($home_phone)
    {
        $this->home_phone = $home_phone;

        return $this;
    }

    /**
     * @internal
     * Sets work_phone
     * @param string $work_phone The work phone number of the customer.
     * @return $this
     */
    public function setWorkPhone($work_phone)
    {
        $this->work_phone = $work_phone;

        return $this;
    }

    /**
     * @internal
     * Sets mobile_phone
     * @param string $mobile_phone The mobile phone number of the customer.
     * @return $this
     */
    public function setMobilePhone($mobile_phone)
    {
        $this->mobile_phone = $mobile_phone;

        return $this;
    }

    /**
     * @internal
     * Sets address
     * @param \Orbipay\PaymentsApi\Client\Wrappers\Address $address address
     * @return $this
     */
    public function setAddress($address)
    {
        $this->address = $address;

        return $this;
    }

    /**
     * @internal
     * Sets custom_fields
     * @param map[string,string] $custom_fields The additional information or meta-information that EBPP can accept, maintain and transmit back to the client. The custom fields need to be configured with EBPP before they can be accepted. EBPP would reject custom fields that are not pre-configured. Please contact <a href = \"mailto: support@billerpayments.com\">support@billerpayments.com</a> for more information on configuring and using custom fields. Only the custom fields enabled for the client, in EBPP, would be accepted in the requests.
     * @return $this
     */
    public function setCustomFields($custom_fields)
    {
        $this->custom_fields = $custom_fields;

        return $this;
    }

    /**
     * @internal
     * Sets customer_reference
     * @param string $customer_reference The unique identifier in the client system for the customer.
     * @return $this
     */
    public function setCustomerReference($customer_reference)
    {
        $this->customer_reference = $customer_reference;

        return $this;
    }

    /**
     * @internal
     * Sets id
     * @param string $id The unique identifier assigned by EBPP to the customer.
     * @return $this
     */
    public function setId($id)
    {
        $this->id = $id;

        return $this;
    }

    /**
     * @internal
     * Sets status
     * @param string $status The status of the customer in EBPP.
     * @return $this
     */
    public function setStatus($status)
    {
        $this->status = $status;

        return $this;
    }

    /**
     * @internal
     * Sets url
     * @param string $url This URL fetches the Customer details.
     * @return $this
     */
    public function setUrl($url)
    {
        $this->url = $url;

        return $this;
    }

    /**
     * @internal
     * Sets comments
     * @param string $comments Comments that can be used to recollect the operation performed on the resource object. API clients need to ensure that no sensitive information is passed in the memo. Alacriti (EBPP) is not responsible for the security of any sensitive information that may be passed as part of the memo.
     * @return $this
     */
    public function setComments($comments)
    {
        $this->comments = $comments;

        return $this;
    }

    /**
     * @internal
     * Sets customer_accounts
     * @param \Orbipay\PaymentsApi\Client\Wrappers\CustomerAccounts $customer_accounts customer_accounts
     * @return $this
     */
    public function setCustomerAccounts($customer_accounts)
    {
        $this->customer_accounts = $customer_accounts;

        return $this;
    }

    /**
     * @internal
     * Sets funding_accounts
     * @param \Orbipay\PaymentsApi\Client\Wrappers\FundingAccounts $funding_accounts funding_accounts
     * @return $this
     */
    public function setFundingAccounts($funding_accounts)
    {
        $this->funding_accounts = $funding_accounts;

        return $this;
    }

    /**
     * @internal
     * Sets payments
     * @param \Orbipay\PaymentsApi\Client\Wrappers\Payments $payments payments
     * @return $this
     */
    public function setPayments($payments)
    {
        $this->payments = $payments;

        return $this;
    }

    /**
     * @internal
     * Sets payment_setups
     * @param \Orbipay\PaymentsApi\Client\Wrappers\PaymentSetups $payment_setups payment_setups
     * @return $this
     */
    public function setPaymentSetups($payment_setups)
    {
        $this->payment_setups = $payment_setups;

        return $this;
    }

    /**
     * @internal
     * Sets audit_info
     * @param \Orbipay\PaymentsApi\Client\Wrappers\AuditInfo $audit_info audit_info
     * @return $this
     */
    public function setAuditInfo($audit_info)
    {
        $this->audit_info = $audit_info;

        return $this;
    }

    /**
     * @internal
     * Sets id_customer
     * @param string $id_customer The unique identifier assigned by EBPP to the customer.
     * @return $this
     */
    public function setIdCustomer($id_customer)
    {
        $this->id_customer = $id_customer;

        return $this;
    }

    /**
     * @var string 
     * $first_name The first name of the customer. Either first_name or last_name is required to create a customer.
     */
    private $first_name;
  
     /**
     * Gets first_name
     *
     * @return string $first_name The first name of the customer. Either first_name or last_name is required to create a customer.
     */
    public function getFirstName()
    {
        return $this->first_name;
    }
    /**
     * @var string 
     * $last_name The last name of the customer. Either first_name or last_name is required to create a customer.
     */
    private $last_name;
  
     /**
     * Gets last_name
     *
     * @return string $last_name The last name of the customer. Either first_name or last_name is required to create a customer.
     */
    public function getLastName()
    {
        return $this->last_name;
    }
    /**
     * @var string 
     * $middle_name The middle name of the customer.
     */
    private $middle_name;
  
     /**
     * Gets middle_name
     *
     * @return string $middle_name The middle name of the customer.
     */
    public function getMiddleName()
    {
        return $this->middle_name;
    }
        
   /**
     * @var string $gender The gender of the customer.
     */
    private $gender;
    
    /**
     * Gets gender
     *
     * @return string $gender The gender of the customer.
     */

    public function getGender()
    {
        return $this->gender;
    }
    
    /**
     * @var string 
     * $date_of_birth The date of birth of the customer. It should be in the full-date format as per ISO8601, namely, YYYY-MM-DD.
     */
    private $date_of_birth;
  
     /**
     * Gets date_of_birth
     *
     * @return string $date_of_birth The date of birth of the customer. It should be in the full-date format as per ISO8601, namely, YYYY-MM-DD.
     */
    public function getDateOfBirth()
    {
        return $this->date_of_birth;
    }
    /**
     * @var string 
     * $ssn The SSN of the customer if the account holder is an individual or the tax ID, if the customer is a business.
     */
    private $ssn;
  
     /**
     * Gets ssn
     *
     * @return string $ssn The SSN of the customer if the account holder is an individual or the tax ID, if the customer is a business.
     */
    public function getSsn()
    {
        return $this->ssn;
    }
    /**
     * @var string 
     * $locale The language in which the customer wants the alerts and notifications from EBPP. This is to be specified in the format,<br><i>< <b>ISO-639-1 language code</b>  >_< <b>ISO ALPHA-2 Country Code</b> ></i>.<br> For e.g., <b>en_US</b> indicates the language preference as US English, which is also the default value.
     */
    private $locale;
  
     /**
     * Gets locale
     *
     * @return string $locale The language in which the customer wants the alerts and notifications from EBPP. This is to be specified in the format,<br><i>< <b>ISO-639-1 language code</b>  >_< <b>ISO ALPHA-2 Country Code</b> ></i>.<br> For e.g., <b>en_US</b> indicates the language preference as US English, which is also the default value.
     */
    public function getLocale()
    {
        return $this->locale;
    }
    /**
     * @var string 
     * $email The email address of the customer to which notifications from EBPP will be sent. This will default to the registered_email while creating a customer.
     */
    private $email;
  
     /**
     * Gets email
     *
     * @return string $email The email address of the customer to which notifications from EBPP will be sent. This will default to the registered_email while creating a customer.
     */
    public function getEmail()
    {
        return $this->email;
    }
    /**
     * @var string 
     * $registered_email The email address of the customer registered with the client. This may be used to uniquely identify the customer if the client is set up accordingly in EBPP.
     */
    private $registered_email;
  
     /**
     * Gets registered_email
     *
     * @return string $registered_email The email address of the customer registered with the client. This may be used to uniquely identify the customer if the client is set up accordingly in EBPP.
     */
    public function getRegisteredEmail()
    {
        return $this->registered_email;
    }
    /**
     * @var string 
     * $home_phone The home phone number of the customer.
     */
    private $home_phone;
  
     /**
     * Gets home_phone
     *
     * @return string $home_phone The home phone number of the customer.
     */
    public function getHomePhone()
    {
        return $this->home_phone;
    }
    /**
     * @var string 
     * $work_phone The work phone number of the customer.
     */
    private $work_phone;
  
     /**
     * Gets work_phone
     *
     * @return string $work_phone The work phone number of the customer.
     */
    public function getWorkPhone()
    {
        return $this->work_phone;
    }
    /**
     * @var string 
     * $mobile_phone The mobile phone number of the customer.
     */
    private $mobile_phone;
  
     /**
     * Gets mobile_phone
     *
     * @return string $mobile_phone The mobile phone number of the customer.
     */
    public function getMobilePhone()
    {
        return $this->mobile_phone;
    }
    /**
     * @var \Orbipay\PaymentsApi\Client\Wrappers\Address 
     * $address address
     */
    private $address;
  
     /**
     * Gets address
     *
     * @return \Orbipay\PaymentsApi\Client\Wrappers\Address $address address
     */
    public function getAddress()
    {
        return $this->address;
    }
    /**
     * @var map[string,string] 
     * $custom_fields The additional information or meta-information that EBPP can accept, maintain and transmit back to the client. The custom fields need to be configured with EBPP before they can be accepted. EBPP would reject custom fields that are not pre-configured. Please contact <a href = \"mailto: support@billerpayments.com\">support@billerpayments.com</a> for more information on configuring and using custom fields. Only the custom fields enabled for the client, in EBPP, would be accepted in the requests.
     */
    private $custom_fields;
  
     /**
     * Gets custom_fields
     *
     * @return map[string,string] $custom_fields The additional information or meta-information that EBPP can accept, maintain and transmit back to the client. The custom fields need to be configured with EBPP before they can be accepted. EBPP would reject custom fields that are not pre-configured. Please contact <a href = \"mailto: support@billerpayments.com\">support@billerpayments.com</a> for more information on configuring and using custom fields. Only the custom fields enabled for the client, in EBPP, would be accepted in the requests.
     */
    public function getCustomFields()
    {
        return $this->custom_fields;
    }
    /**
     * @var string 
     * $customer_reference The unique identifier in the client system for the customer.
     */
    private $customer_reference;
  
     /**
     * Gets customer_reference
     *
     * @return string $customer_reference The unique identifier in the client system for the customer.
     */
    public function getCustomerReference()
    {
        return $this->customer_reference;
    }
    /**
     * @var string 
     * $id The unique identifier assigned by EBPP to the customer.
     */
    private $id;
  
     /**
     * Gets id
     *
     * @return string $id The unique identifier assigned by EBPP to the customer.
     */
    public function getId()
    {
        return $this->id;
    }
        
   /**
     * @var string $status The status of the customer in EBPP.
     */
    private $status;
    
    /**
     * Gets status
     *
     * @return string $status The status of the customer in EBPP.
     */

    public function getStatus()
    {
        return $this->status;
    }
    
    /**
     * @var string 
     * $url This URL fetches the Customer details.
     */
    private $url;
  
     /**
     * Gets url
     *
     * @return string $url This URL fetches the Customer details.
     */
    public function getUrl()
    {
        return $this->url;
    }
    /**
     * @var string 
     * $comments Comments that can be used to recollect the operation performed on the resource object. API clients need to ensure that no sensitive information is passed in the memo. Alacriti (EBPP) is not responsible for the security of any sensitive information that may be passed as part of the memo.
     */
    private $comments;
  
     /**
     * Gets comments
     *
     * @return string $comments Comments that can be used to recollect the operation performed on the resource object. API clients need to ensure that no sensitive information is passed in the memo. Alacriti (EBPP) is not responsible for the security of any sensitive information that may be passed as part of the memo.
     */
    public function getComments()
    {
        return $this->comments;
    }
    /**
     * @var \Orbipay\PaymentsApi\Client\Wrappers\CustomerAccounts 
     * $customer_accounts customer_accounts
     */
    private $customer_accounts;
  
     /**
     * Gets customer_accounts
     *
     * @return \Orbipay\PaymentsApi\Client\Wrappers\CustomerAccounts $customer_accounts customer_accounts
     */
    public function getCustomerAccounts()
    {
        return $this->customer_accounts;
    }
    /**
     * @var \Orbipay\PaymentsApi\Client\Wrappers\FundingAccounts 
     * $funding_accounts funding_accounts
     */
    private $funding_accounts;
  
     /**
     * Gets funding_accounts
     *
     * @return \Orbipay\PaymentsApi\Client\Wrappers\FundingAccounts $funding_accounts funding_accounts
     */
    public function getFundingAccounts()
    {
        return $this->funding_accounts;
    }
    /**
     * @var \Orbipay\PaymentsApi\Client\Wrappers\Payments 
     * $payments payments
     */
    private $payments;
  
     /**
     * Gets payments
     *
     * @return \Orbipay\PaymentsApi\Client\Wrappers\Payments $payments payments
     */
    public function getPayments()
    {
        return $this->payments;
    }
    /**
     * @var \Orbipay\PaymentsApi\Client\Wrappers\PaymentSetups 
     * $payment_setups payment_setups
     */
    private $payment_setups;
  
     /**
     * Gets payment_setups
     *
     * @return \Orbipay\PaymentsApi\Client\Wrappers\PaymentSetups $payment_setups payment_setups
     */
    public function getPaymentSetups()
    {
        return $this->payment_setups;
    }
    /**
     * @var \Orbipay\PaymentsApi\Client\Wrappers\AuditInfo 
     * $audit_info audit_info
     */
    private $audit_info;
  
     /**
     * Gets audit_info
     *
     * @return \Orbipay\PaymentsApi\Client\Wrappers\AuditInfo $audit_info audit_info
     */
    public function getAuditInfo()
    {
        return $this->audit_info;
    }
    /**
     * @var string 
     * $id_customer The unique identifier assigned by EBPP to the customer.
     */
    private $id_customer;
  
     /**
     * Gets id_customer
     *
     * @return string $id_customer The unique identifier assigned by EBPP to the customer.
     */
    public function getIdCustomer()
    {
        return $this->id_customer;
    }

    
     /**
     * Customer constructor.
     * @param string $id_customer 
     */
    public function __construct($id_customer = null)
    {
        $this->id_customer = $id_customer;
		$this->id = $id_customer;
    }
       
    public function forClient($client_key = null)
    {
		parent::forClient($client_key);
        
        return $this; 
    }
    public function videChannel($channel = null)
    {
		parent::videChannel($channel);
        
        return $this; 
    }
    public function withDetails($first_name = null, $last_name = null, $middle_name = null, $date_of_birth = null, $ssn = null, $gender = null, $registered_email = null)
    {
        			$this->first_name = $first_name;
        		
        

        			$this->last_name = $last_name;
        		
        

        			$this->middle_name = $middle_name;
        		
        

        			$this->date_of_birth = $date_of_birth;
        		
        

        			$this->ssn = $ssn;
        		
        

        			$this->gender = $gender;
        		
        

        			$this->registered_email = $registered_email;
        		
        

        
        return $this; 
    }
    public function withReference($customer_reference = null)
    {
        			$this->customer_reference = $customer_reference;
        		
        

        
        return $this; 
    }
    public function withStatus($status = null)
    {
        			$this->status = $status;
        		
        

        
        return $this; 
    }
    public function withContactInfo($email = null, $home_phone = null, $work_phone = null, $mobile_phone = null)
    {
        			$this->email = $email;
        		
        

        			$this->home_phone = $home_phone;
        		
        

        			$this->work_phone = $work_phone;
        		
        

        			$this->mobile_phone = $mobile_phone;
        		
        

        
        return $this; 
    }
    public function withLocale($locale = null)
    {
        			$this->locale = $locale;
        		
        

        
        return $this; 
    }
    public function withAccounts($customer_accounts = null)
    {
        			$this->customer_accounts = $customer_accounts;
        		
        

        
        return $this; 
    }
    public function withAddress($address = null)
    {
        			$this->address = $address;
        		
        

        
        return $this; 
    }
    public function withCustomFields($custom_fields = null)
    {
				if(isset($custom_fields)){
					if(count($custom_fields) == 0){
						$this->custom_fields = null;
					}
					else{
        				$this->custom_fields = $custom_fields;
					}
        		}else{
        			$this->custom_fields = $custom_fields;
        		}
        		
        

        
        return $this; 
    }
    public function withMemo($comments = null)
    {
        			$this->comments = $comments;
        		
        

        
        return $this; 
    }
    
	/**
     * createdBy
     *
     * @param $requestor
     * @param $requestor_type
     * @return Customer
     */
    public function createdBy($requestor = null , $requestor_type = null )
    {
       $this->requestedBy( $requestor, $requestor_type);
       return $this;
    }

	/**
     * retrievedBy
     *
     * @param $requestor
     * @param $requestor_type
     * @return Customer
     */
    public function retrievedBy($requestor = null , $requestor_type = null )
    {
       $this->requestedBy( $requestor, $requestor_type);
       return $this;
    }

	/**
     * updatedBy
     *
     * @param $requestor
     * @param $requestor_type
     * @return Customer
     */
    public function updatedBy($requestor = null , $requestor_type = null )
    {
       $this->requestedBy( $requestor, $requestor_type);
       return $this;
    }

    
	/**
     * create
     *
     * @param InvocationContext $invocationContext
     * @param string $liveMode
     * @param string $apiEndPoint
     * @return Customer
     * @throws OrbipayApiException
     */
    public function create(InvocationContext $invocationContext, $liveMode = null,  $apiEndPoint = null)
    {
     	try {
	    	$this->withContext($invocationContext, $liveMode, $apiEndPoint);
	    	$handler = new  \Orbipay\PaymentsApi\Client\Handlers\CreateCustomerHandler();
	        return $handler->process($this);
		} catch (OrbipayApiException $e) {
            throw $e;
        } catch (\Exception $e) {
             $logger = LoggerFactory::getLogger(__CLASS__);
             $logger->error($e->getMessage());
            throw new OrbipayApiException($e->getMessage(), $e->getCode(), $e);
        }
    }
	/**
     * get
     *
     * @param InvocationContext $invocationContext
     * @param string $liveMode
     * @param string $apiEndPoint
     * @return Customer
     * @throws OrbipayApiException
     */
    public function get(InvocationContext $invocationContext, $liveMode = null,  $apiEndPoint = null)
    {
     	try {
	    	$this->withContext($invocationContext, $liveMode, $apiEndPoint);
	    	$handler = new  \Orbipay\PaymentsApi\Client\Handlers\GetCustomerHandler();
	        return $handler->process($this);
		} catch (OrbipayApiException $e) {
            throw $e;
        } catch (\Exception $e) {
             $logger = LoggerFactory::getLogger(__CLASS__);
             $logger->error($e->getMessage());
            throw new OrbipayApiException($e->getMessage(), $e->getCode(), $e);
        }
    }
	/**
     * update
     *
     * @param InvocationContext $invocationContext
     * @param string $liveMode
     * @param string $apiEndPoint
     * @return Customer
     * @throws OrbipayApiException
     */
    public function update(InvocationContext $invocationContext, $liveMode = null,  $apiEndPoint = null)
    {
     	try {
	    	$this->withContext($invocationContext, $liveMode, $apiEndPoint);
	    	$handler = new  \Orbipay\PaymentsApi\Client\Handlers\UpdateCustomerHandler();
	        return $handler->process($this);
		} catch (OrbipayApiException $e) {
            throw $e;
        } catch (\Exception $e) {
             $logger = LoggerFactory::getLogger(__CLASS__);
             $logger->error($e->getMessage());
            throw new OrbipayApiException($e->getMessage(), $e->getCode(), $e);
        }
    }
    
    
	/**
     * Specify data which should be serialized to JSON
     * @link http://php.net/manual/en/jsonserializable.jsonserialize.php
     * @return mixed data which can be serialized by <b>json_encode</b>,
     * which is a value of any type other than a resource.
     * @since 5.4.0
     */
    public function jsonSerialize()
    {
        return CommonUtil::json_serialize(get_object_vars($this) , parent::jsonSerialize());
    }
}


